#! /bin/csh -f
#   setupsort script
#   MT version  Mon Apr  7 11:07:53 BST 1997
#
#   $1=full pathname of sort setup file (file coded in Eurogam sort language)
#   $2=full pathname of sort setup directory (to contain compiled program)
#   $3=data format (eurogam....)
#   $4=userlib
#
# set verbose mode if trace defined and non-zero
if (!($?TRACE)) then
   set trace=0
else
   set trace=$TRACE
endif
if ($trace) echo "trace=$trace"
#
if (!($?MIDASBASE)) then
   setenv MIDASBASE /MIDAS
endif
if (!($?MTSORTBASE)) then
   setenv MTSORTBASE $MIDASBASE/MTsort
endif
#
if($1 == "") then
echo Sort data file not specified
exit(1)
endif
#
if(-r $1) then
else
echo Sort data file not readable
exit(1)
endif
#
if($2 == "") then
echo Sort setup directory not specified
exit(1)
endif
#
if(-e $2) then
else
  echo "Creating $2..."
  mkdir $2
  if ($status) then
    echo "Sort setup error: failed to create directory $2"
    exit(1)
  endif
endif
if(-d $2) then
else
  echo "Sort setup error: $2 is not a directory"
  exit(1)
endif
#
set version=unix
set format=""
if($3 == "eurogam" || $3 == "nsf") then
set format=$3
else
set format="eurogam"
endif
#
echo "Sort setup ($format format events, $version version) from file=$1" 
#
pushd $2 > /dev/null
# save sortlog and delete all other files
if(-f sortlog) then
  mv sortlog .sortlog
# delete maps, etc through spectrum zerver
  MTsortMapDelete
  rm -f *
  mv .sortlog sortlog
else
# delete maps, etc through spectrum zerver
  MTsortMapDelete
  rm -f *
endif
cp $1 sortfile
MTsort -h$version -f$format -l$2 <sortfile >&! sort.out
set sstatus=$status
cat sort.out
if ($sstatus) then
  if ($sstatus == 1) then
    exit(1)
  endif
endif
if ($sstatus>4 || $sstatus<0) then
echo Sort compilation failed with code $sstatus
exit(3)
endif
cat cdata.sort ccode.sort >&! sort.c
echo
# -DLITTLE_ENDIAN -D_NOSORTMAIN2 removed from Linux gcc line
if(`uname` == "Linux") then
  gcc -shared -g -fPIC -DPTHREADS -DLINUX -I$MTSORTBASE/include sort.c $4 -o MTsortEvent.so >&! complist
  set sstatus=$status
else
  gcc -G -g -O2 -I$MTSORTBASE/include sort.c $4 -o MTsortEvent.so >&! complist
  set sstatus=$status
endif
if($sstatus) goto compile_error
echo "Event shared object C compilation succeeded"
echo OK.v4 >&! setup_status
goto make_maps
echo "C compilation succeeded"
make_maps:

if(-e $2/.mapfiles) then
 echo Map file generation delayed until run-time ...
else
 # create maps
 MTmapmake <mapinfo
 if($status) goto mapmake_error
endif

# create gain arrays
MTgainmake
if($status) goto gainmake_error
# generate rundata from tape info
if ($trace) echo "generate runfiles info..."
MTrunfiles < runinfo
if($status) goto runfiles_error
tidy_up:
if ($trace) echo "tidy up..."
#mv initdata.sort initdata
#rm -f *.sort
if ($trace) echo "remove discfile, mapspace..."
rm -f discfile mapspace
if ($trace) echo "save $1 to infile..."
cp $1 infile
if ($trace) echo "before popd: cwd=$cwd..."
popd > /dev/null
if ($trace) echo "after popd: cwd=$cwd..., exit(0)"
exit(0)
compile_error:
echo Sort setup C compilation failed ... examine complist 
#rm sort.*
popd > /dev/null
exit(4)
mapmake_error:
echo Sort setup error creating maps
rm -f *.sort
#rm -f sort.*
popd > /dev/null
exit(5)
gainmake_error:
echo Sort setup error creating gain arrays
rm -f *.sort
#rm -f sort.*
popd > /dev/null
exit(6)
runfiles_error:
echo Sort setup error creating runfiles info
rm -f *.sort
#rm -f sort.*
popd > /dev/null
exit(7)
