set SpectrumServerHost localhost

package require ServerAccess
package require SpectrumClient

#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		sod.tcl (sort diagnostics)       		#
#									#
#	Author:	        John Cresswell		  	                #
#									#
#	Last revision:	1/4/2014                                        #
#									#
#########################################################################

set frame SOD
set frametitle "MTsort Diagnostics"
set icontitle "Diagnostics"

set thismachine [exec uname -n]
set sortmachine $thismachine
set test_result 0

set started 0

if { ![info exists env(MTsortVersion)]} {set mtsortversion ""} {set mtsortversion "$env(MTsortVersion)"} 

proc click arg {
   global debug

   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

       MACHINE   {click-machine $v}
       REPAIR    {click-repair}
       REDISPLAY {redisplay}
       MYHELP    {click-help}
       default   {}
   }
}


proc put-galley text {
   :modify SORTLOG :append T :strings "$text\n" :see end
   }

proc make-frame {} {
    global sortmachine test_result

    :set footer_position center
    :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
	:icon-name "[self icon-title]"
    

    :panel PANEL1 :layout T :padx 10 :pady 10
    :layout H; :row-gap 10

    :galley SORTLOG :rows 12 :cols 60 :x 0 :deaf T

    put-galley "Sort machine $sortmachine"

    set test_result  [run-tests]
   
    :panel PANEL2 :layout T  :padx 10
    :layout H; :col-gap 20
    :button REPAIR  :label "Try Repair session"
    :button REDISPLAY  :label "Redisplay"

#    :button MYHELP  :label "Help" :command click-help
  :show T

}

proc sod.read-specpath f {
    global specpath
    set specpath $f
}

proc run-tests {} {
    global shmtotal shmused instancelist specpath

    do-shm
    put-galley "Shared memory:  Total $shmtotal MBytes   Used $shmused Mbytes"
    
     if { ! [is-shmsas-running]} {
	spectrum_server_not_running
	return 1
    }

    put-galley "Shmsas Spectrum server running ..."
    
    set dirs ""
    set z 0
   set z [catch {eg "(spectrum :path '/sort' ?names)"} dirs]
   if {$z} {
	switch $dirs {
	    0x30004 {put-galley "Invalid capability - free and claim apparatus  sort.localhost"}
	    default {put-galley "Error $dirs"}
	}
	return 1
    }
    

   inform soc call-back sod sod.read-specpath get-specpath

    if {$z} {
	put-galley "Sort run window not responding"
	return 2
    } {
	if {$specpath == ""} {put-galley "Spectrum path not defined"}
    }

    put-galley "Spectrum path $specpath"
    put-galley "Shared memory directories ..."

    set sasport [inform sos get-sasport]
    
    set n 0
    foreach p $dirs {
	set t [first $p]
	if {$t != 2} {continue}
	set s [tail $p]
	if {$s == $specpath} {set c "... Currently set"} {set c ""}
	set m ""
	set z [catch {exec ls -al /dev/shm/shmspectra$sasport.$n} m]
	if {[first $m] != "ls:"} {set mm [expr [fifth $m] / 1000000]} { set mm 0}
	put-galley "     /sort/$s $c Using $mm Mbytes"
	incr n
    }
    if {!$n} {
    put-galley "     none found ... created on GO using defined Spectrum Path"
    }

    put-galley "Current MTsort instances ..."
    
    do-instances
    set n 0
    set ports {}
    foreach p $instancelist {
	set t [third $p]
	set z [catch {exec cat [last $p]} d]
	lappend ports [string range $p end-4 end]
	puts "$t $d ... [last $p]"
	incr n
	
	put-galley "     $d   $t  port  [string range $p end-4 end]"
    }
    if {!$n} {
	put-galley "     none found - try Sort Reload?"
	put-galley "If that fails, then look for error in Scheduler or Run windows"
    }
    if {$n > 1} {
	put-galley " More than one sort instance found ... on ports $ports"
    }

# Is sort running? ...

    if {$n == 1} {
	put-galley "One sort instance found ... port $ports"
	set nn 0
	set z [catch {exec /bin/ps -ef | grep $ports | grep MTsortSchedule} m]
	if {$z} {put-galley "MTsort Scheduler not running - Sort Reload?"} {incr nn}
	
	set z [catch {exec /bin/ps -ef | grep $ports | grep MTsortEvent} m]
	if {$z} {put-galley "MTsort Event task not running - See Scheduler log"} {incr nn}
	set z [catch {exec /bin/ps -ef | grep $ports | grep MTsortInput} m]
	if {$z} {put-galley "MTsort Input task not running"} {incr nn}
	set z [catch {exec /bin/ps -ef | grep $ports | grep MTsortOutput} m]
	if {$z} {put-galley "MTsort Output task not running"} {incr nn}

	if {$nn == 4} {put-galley "All sort tasks are running"}
    }

    return 0
}

proc is-shmsas-running {} {
    global env
    switch $env(os) {
	SunOS {set flags -ef}
	Linux     -
	default {set flags -ef}
    }
    set z [catch {exec /bin/ps $flags | grep shmsas | grep MIDAS | grep $env(USER) | grep -v /MIDAS/sas} m]
    if {$z} {return 0}
    return 1
}

proc spectrum_server_not_running {} {
    put-galley "Shmsas Spectrum server not running ... Try Repair"
    return 0
}

proc do-shm {} {
   global shmtotal shmused shmlist
    set l [exec df /dev/shm]
    set s [string first tmpfs $l]
    set l [string range $l $s end]
  
    set shmtotal [expr [second $l] /1024]
    set shmused [expr [third $l] /1024]

    set z [catch {exec ls -al {*}[glob /dev/shm/shmspectra*]} m]
    set shmlist $m

}

proc do-instances {} {
   global instancelist

    set z [catch {exec ls -al [glob /tmp/mtsort*]} i]

#Calculate number of tasks running ...

    if {$z} {
	set instancelist {}
    } {
	set instancelist [split $i \n]
    }
}

proc redisplay {} {make-frame}

# Rerun shmsas and free/claim apparatus
proc click-repair {} {
    global test_result
    frame-busy

    if {$test_result == 1} {
	inform sos click-run-shmsas
	after 1000

	if { ! [is-shmsas-running]} {
	    spectrum_server_not_running
	    frame-idle
            return
	}

	inform sos quiet-free-claim-apparatus
	redisplay
    }

    frame-idle
}

proc click-help {} {
    inform help html_show {"" $env(MTSORTBASE)/manual/MTsort.html}
   }

# Requires htmlviewer.patch.tcl  to work ...
proc click-help {} {
    global env
    inform htmlviewer startfile $env(MTSORTBASE)/manual/MTsort.html
   }

proc finish {} {
   global started
   set started 0
   }
