#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		soi-CaenDigitiserConfig.tcl		                #
#									#
#	Author(s):	John Cresswell           			#
#									#
#	Last revision:	24/02/2009					#
#									#
#########################################################################

set frametitle "MTsort CaenDigitiser Online Sort Input Handler Config"

set currentname     0
set currentlabel    0
set currentflag     0
set currentgroup    0
set currentitem     0

# 0=CaenDigitiser EB Event  1=Event-by-timestamp jump
set eventmode 0
set clockjump 100


set z [catch {set env(os)} ostype]
if {$z} {set ostype xxx}
if {$ostype == "SunOS"} {
  set devlist {"Not_Active" "Net" "Other?"}
  } {
  set devlist {"Not_Active" "Net" "Other?"}
  }


#------------------------------------------------------------------------------

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      ADD                    {click-add}
      INSERT                 {click-insert}
      REPLACE                {click-replace}
      REMOVE                 {click-remove}
      SAVE                   {click-save}

      RUNLIST                {click-runlist $c $v}
      EVENTMODE              {click-eventmode $v}
      PARAM                  {click-paramname $v}
      LABEL                  {click-label $v}
      FLAG                   {click-flag $v}
      GROUP                  {click-group $v}
      ITEM                   {click-item $v}
      JUMP                   {click-tsjump $v}

      SORT-RUN-FRAME         {click-sort-run-frame}
      MYHELP                   {click-help}
      REFRESH                {click-refresh}

      INPUT-HANDLER          {click-input-handler $v}

      STREAM                 {click-stream $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug env
   global eventmodemenustrings
   global handlerlist handlerselected handlerloaded status
   global devlist debugmenustrings
   global runfilelist runfileheader

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}
   if {$debug} {puts stdout "...devlist=$devlist"}

  set vx1 10c
  set vx2 19c
  set vx3 300
  set twid1 15
  set twid2 6
  set twid3 30

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :button SAVE :label "Save"
      :button SORT-RUN-FRAME :label "Sort Run Frame" :x $vx3
      :button MYHELP :label "Help"
      :button REFRESH :label "Refresh"

   :panel RUNFILES :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

      :text JUMP  :label "Event Split Clks:" :w $twid2 :x 280

  :panel OPTIONS  :layout L :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 10
      :message INPUT-HANDLER :label "Input handler:    $handlerloaded" :bold T
      :text INPUT-HANDLER-STATUS :label "Status:" :value $status :bold T \
         :w $twid1 :valuex $vx2 :x 300
      [:path INPUT-HANDLER-STATUS].entry configure -bd 0
   :show T
}

proc paint-handler-status {} {
   global status basedir
   set z [mtsort $basedir "i state" status 1]
   :modify INPUT-HANDLER-STATUS :value $status
   }

proc repaint {} {
   global basedir handlerselected handlerloaded
   global devlist device1selected device2selected
   global device2activestates device2active
   global tracelist tracelevel

   check-handler
   :modify INPUT-HANDLER :label "Input handler:    $handlerloaded"
   paint-handler-status

   paint-runfiles
   }

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	set f ""
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	lappend buf $t
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

proc read-runinfo {} {
   global runfilelist runfilename eventmode  clockjump
   set runfilelist {}
   set i 0
   set z 0
   while {$z==0} {
      set i [expr $i + 1]
      set z [read-file $runfilename]
      if {$z!=""} then {
	  foreach l $z {
	      if { [isComment? $l] } {
		  if {[string range $l 0 7] == "#command"} {
		      scan $l "%s %d %d" s eventmode clockjump
		  }
		  if {$clockjump == ""} {set clockjump 100}
	      }
	  }
      }
   }
   paint-runfiles
}


proc paint-runfiles {} {
   global clockjump
    :modify JUMP :value $clockjump
}
                 
proc click-stream e {
   global basedir
   global device1selected devlist
   frame-busy
   set device1selected $e
   set d [lindex $devlist $e]
    set m ""
   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device $d" m 1]
   if {$z!=0} then {set-footer $m; set device1selected -1} \
   else {
      clear-footer
      }
   repaint
   frame-idle
   }

proc click-eventmode {s} {
   global eventmode eventmodemenustrings
   set eventmode $s
   }

proc click-tsjump {v} {
   global clockjump
   set clockjump $v
   }

proc click-paramname v {
    global currentname currentlabel
    set currentname [string toupper $v]
    if {$currentname == ""} {set currentname $currentlabel}
   }

proc click-label v {
    global currentname currentlabel
    set currentlabel [string toupper $v]
    if {$currentname == ""} {set currentname $v}
   }

proc click-flag v {
    global currentname currentflag
    set currentflag [string toupper $v]
    if {$currentname == ""} {set currentname $v}
   }

proc click-group v {
    global currentgroup currentitem
    set currentgroup [string toupper $v]
    if {$currentitem == ""} {set currentitem 0}
   }

proc click-item v {
   global currentitem
   set currentitem [string toupper $v]
   }

proc click-add {} {
   }

proc click-insert {} {
   }

proc click-replace {} {
   }

proc click-remove {} {
   }

proc click-save {} {
   global basedir sortdir runfilelist runfilename eventmode clockjump
   frame-busy

    set w [list "\#command $eventmode $clockjump"]

    set z [write-file $runfilename $w]
    if {$z!=0} then {
         display-midas-error-report self "" "Error saving runfiles" $m
         frame-idle
         return
         }
   frame-idle
   }


proc click-refresh {} {
   frame-busy
   clear-footer
   read-runinfo
   paint-runfiles
   repaint
   frame-idle
   }

proc init-values {} {
   global debug
   global basedir tracelevel tracelist
   global devlist device1selected device2selected
   global runfilename

   read-handler-directory

    set m ""
   set z [mtsort $basedir "i trace ?" m 1]
   set tracelevel $m
   if {$debug} then {puts "soi init: trace $z $m"}

   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device1selected [lsearch $devlist $m] } \
                        else { set device1selected 0 }
   if {$debug} then {puts "soi init: device1 $z $m"}

   set z [mtsort $basedir "i stream 2" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device2selected [lsearch $devlist $m] } \
                        else { set device2selected 0 }
   if {$debug} then {puts "soi init: device2 $z $m"}

   set runfilename $basedir/.mtsort.cf

   return 0
   }


proc start-handler args {
   global started
   if {![init-values]} then { 
      make-frame
      update idletasks
      click-refresh
      set started 1
      }
   }
