#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		soi-In2p3Config.tcl		                #
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	10/06/02					#
#									#
#########################################################################

set frametitle "MTsort In2p3 Sort Input Handler Config"

set currentmode 0
set currentvariable  0

set modemenustrings "old variable"

set runfileheader "Handler Parameters ..."

set z [catch {set env(os)} ostype]
if {$z} {set ostype xxx}
if {$ostype == "SunOS"} {
  set devlist {"Not_Active" "Net" "Other?"}
  } {
  set devlist {"Not_Active" "Net" "Other?"}
  }

#------------------------------------------------------------------------------

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      SAVE                   {click-save}

      SORT-RUN-FRAME         {click-sort-run-frame}
      MYHELP                   {click-help}
      REFRESH                {click-refresh}

      MODE                  {click-mode $v}
      VARIABLE              {click-variable $v}

      INPUT-HANDLER          {click-input-handler $v}

      STREAM                 {click-stream $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug
   global env
   global handlerlist handlerselected handlerloaded status
   global devlist debugmenustrings modemenustrings
   global runfileheader currentmode

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}
   if {$debug} {puts stdout "...devlist=$devlist"}

  set vx1 12c
  set vx2 20c
  set vx3 300
  set twid1 15
  set twid2 60
  set twid3 30

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :button SAVE :label "Save"
      :button SORT-RUN-FRAME :label "Sort Run Frame" :x $vx3
      :button MYHELP :label "Help"
      :button REFRESH :label "Refresh"
   :panel RUNFILES :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
      :message LIST-HEADER :label "$runfileheader" :bold T
     :next-row T
      :choice-stack MODE :valuex $vx2 :label "Mode" :strings $modemenustrings
     :next-row T
      :text VARIABLE :label "Number of Fixed words:" :w 10 :valuex $vx2
  :panel OPTIONS  :layout L :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 10
      :message INPUT-HANDLER :label "Input handler:    $handlerloaded" :bold T
      :text INPUT-HANDLER-STATUS :label "Status:" :value $status :bold T \
         :w $twid1 :valuex $vx2 :x 300
      [:path INPUT-HANDLER-STATUS].entry configure -bd 0
     :next-row T
      :message STREAM :label "Stream 1" :bold T
   :show T
}

proc paint-handler-status {} {
   global status basedir currentmode currentvariable
   set z [mtsort $basedir "i state" status 1]
   :modify INPUT-HANDLER-STATUS :value $status
    :modify MODE :value $currentmode
   :modify VARIABLE :value $currentvariable
   }

proc repaint {} {
   global basedir handlerselected handlerloaded
   global devlist device1selected device2selected
   global device2activestates device2active
   global tracelist tracelevel

   check-handler
   :modify INPUT-HANDLER :label "Input handler:    $handlerloaded"
   paint-handler-status
   }

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	display-midas-error-report self "" $m
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
#	if { ! [isComment? $t] } { lappend buf $t }
	lappend buf $t
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

proc read-runinfo {} {
   global basedir runfilename
   global currentmode currentvariable modemenustrings
   set i 0
   set z 0
   while {$z==0} {
      set i [expr $i + 1]
      set z [read-file $runfilename]
      if {$z!=""} then {
	  set l [first $z]
	  set mode [second $l]
	  set currentmode [lsearch $modemenustrings $mode]
	  set l [second $z]
	  set currentvariable [second $l]
      }
   }
   }

proc click-stream e {
   }

proc click-mode v {
   global currentmode
   set currentmode [string tolower $v]
   }

proc click-variable e {
   global currentvariable
   set currentvariable $e
   }

proc click-save {} {
   global basedir sortdir runfilelist runfilename
   global currentmode currentvariable modemenustrings
   frame-busy

    set w ""
    lappend w "mode [lindex $modemenustrings $currentmode]"
    lappend w "variable $currentvariable"
    set z [write-file $runfilename $w]
    if {$z!=0} then {
         display-midas-error-report self "" "Error saving runfiles" $m
         frame-idle
         return
         }
   frame-idle
   }

proc click-refresh {} {
   frame-busy
   clear-footer
   read-runinfo
   repaint
   frame-idle
   }

proc init-values {} {
   global debug
   global basedir tracelevel tracelist
   global devlist device1selected device2selected
   global runfilename

   read-handler-directory

    set m ""
   set z [mtsort $basedir "i trace ?" m 1]
   set tracelevel $m
   if {$debug} then {puts "soi init: trace $z $m"}

   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device1selected [lsearch $devlist $m] } \
                        else { set device1selected 0 }
   if {$debug} then {puts "soi init: device1 $z $m"}

   set z [mtsort $basedir "i stream 2" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device2selected [lsearch $devlist $m] } \
                        else { set device2selected 0 }
   if {$debug} then {puts "soi init: device2 $z $m"}

   set runfilename $basedir/.mtsort.cf

   return 0
   }


proc start-handler args {
   global started
   if {![init-values]} then { 
      make-frame
      update idletasks
      click-refresh
      set started 1
      }
   }
