##########################################################################
#									 #
#	MIDAS General sort package					 #
#									 #
#	File:	        soi-Remote.tcl (sort input remote handler setup) #
#									 #
#	Author(s):      John Cresswell/Janet Sampson           		 #
#									 #
#	Last revision:  24/02/09				         #
#									 #
##########################################################################
#
# Remote version justs adds port choice to Online version
#

set frametitle "MTsort Input Remote Online Handler Control"

set currentvolume "NET"
set currentfile "NET"
set currentstart 0
set currentend 0

set currentport 10305
set portfilename ""

set z [catch {set env(os)} ostype]
if {$z} {set ostype xxx}
if {$ostype == "SunOS"} {
  set devlist {"Not_Active" "Net" "Other?"}
  } {
  set devlist {"Not_Active" "Net" "Other?"}
  }

#------------------------------------------------------------------------------

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      ADD                    {click-add}
      REPLACE                {click-replace}
      REMOVE                 {click-remove}
      SAVE                   {click-save}

      SORT-RUN-FRAME         {click-sort-run-frame}
      MYHELP                   {click-help}
      REFRESH                {click-refresh}

      DEBUG                  {click-debug $v}
      CONFIG                 {click-config $v}
      PORT                   {click-port $v}

      GO                     {click-handler-go}
      STOP                   {click-handler-stop}

      STREAM                 {click-stream $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug env
   global handlerlist handlerselected handlerloaded status
   global devlist debugmenustrings
   global runfilelist currentport

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}
   if {$debug} {puts stdout "...devlist=$devlist"}

  set vx1 12c
  set vx2 22c
  set vx3 300
  set twid1 15
  set twid2 10

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :choice-stack DEBUG :label "Debug" :strings $debugmenustrings
      :button CONFIG :label "Config" :x 150
      :button SORT-RUN-FRAME :label "Sort Run Frame" :x $vx3
      :button MYHELP :label "Help"
      :button REFRESH :label "Refresh"
  :panel OPTIONS  :layout L :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 10
      :message INPUT-HANDLER :label "Input handler:    $handlerloaded" :bold T
     :next-row T
      :button GO   :label Go
      :button STOP :label Stop
     :next-row T
      :message STREAM :label "Stream 1" :bold T
      :text PORT :label "Port:" :w $twid2 :valuex $vx1 :value $currentport
   :panel INFO  :layout L :padx 10 :pady 10
      :layout V; :row-gap 20; :col-gap 10
      :text INPUT-HANDLER-STATUS :label "Status:" :value $status :bold T \
         :w $twid1 :valuex $vx2
      [:path INPUT-HANDLER-STATUS].entry configure -bd 0
      :text STATISTICS-TOTAL :label "STATISTICS    total:" :value 0 :bold T \
         :w $twid1 :valuex $vx2
      [:path STATISTICS-TOTAL].entry configure -bd 0
      :text STATISTICS-CURRENT :label "current:" :value 0 :bold T \
         :w $twid1 :valuex $vx2
      [:path STATISTICS-CURRENT].entry configure -bd 0
   :show T
}

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	set f ""
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { ! [isComment? $t] } { lappend buf $t }
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

proc paint-handler-status {} {
   global status basedir
   set z [mtsort $basedir "i state" status 1]
   :modify INPUT-HANDLER-STATUS :value $status
   }

proc repaint {} {
   global basedir handlerselected handlerloaded
   global devlist device1selected device2selected
   global device2activestates device2active
   global tracelist tracelevel

   check-handler
   :modify INPUT-HANDLER :label "Input handler:    $handlerloaded"
   paint-handler-status

   set statistics2 "Unknown q Unknown q"
    set m ""
   if { $device2selected != 0} then {
      set z [mtsort $basedir "i stream 2" m 1]
      set z [mtsort $basedir "i stats" m 1]
      if {$z==0} then { set statistics2 [string range $m 26 end] }
      set z [mtsort $basedir "i stream 1" m 1]
      }
   set z [mtsort $basedir "i stats" m 1]
   if {$z} then { set statistics1 "Unknown q Unknown q"} \
   else { set statistics1 [string range $m 26 end] }
   set statistics2 "0 block 0 current"

   if {$device2selected > 0} then {
#      xv "(SOI-STATS-DEV2-VALUE :label '[third $statistics2]')"
#      xv "(SOI-PANEL1A :show F) (SOI-PANEL1B :show T)"
      }

   :modify STATISTICS-TOTAL :value [first $statistics1]
   :modify STATISTICS-CURRENT :value [third $statistics1]
   :modify DEBUG :value $tracelevel

   paint-runfiles
   }

proc set-runinfo {} {
   global runfilename runfilelist status

   set runfilelist {}
     set l "NET NET 0 0"
    lappend runfilelist [format-runfile $l]
    set runfileselected [expr [llength $runfilelist] - 1]
# If not going ...    
    if {$status != "Going"} {click-save}
   }

proc read-runinfo {} {
   global basedir runfilename runfilelist
    set-runinfo
    read-portinfo
   paint-runfiles
   }

proc read-portinfo {} {
    global portfilename
    global currentport

    set z [catch {set l [read-file $portfilename]} m]
    if {!$z} {
	if {$l!=""} {
	    set currentport  $l
	}
    } {
	set currentport 10305
    }

    paint-runfiles
}

proc format-runfile {r} {
   scan $r "%s %s %d %d" s1 s2 i1 i2
   set r [format "%6s %-30s %12d %12d" $s1 $s2 $i1 $i2]
   return $r
   }

proc paint-runfiles {} {
   global currentport
   :modify PORT :value $currentport
   }
                 
proc click-stream e {
   global basedir
   global device1selected devlist
   frame-busy
   set device1selected $e
   set d [lindex $devlist $e]
    set m ""
   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device $d" m 1]
   if {$z!=0} then {set-footer $m; set device1selected -1} \
   else {
      clear-footer
      }
   repaint
   frame-idle
   }

proc click-port v {
   global currentport
   set currentport [string toupper $v]
    click-save
   }

proc click-save {} {
   global portfilename
   global currentport
   frame-busy

   set i 0
   set z 0
   set m ""
   foreach x $runfilelist {
      set i [expr $i + 1]
      set z [mtsort $basedir "i runinfo $i $x" m 1]
      if {$z!=0} then {
         display-midas-error-report self "" "Error saving runfiles:\n" $m
         frame-idle
         return
         } \
      else {
         set z [mtsort $basedir "i runinfo $i eof" m 1]
         set-footer $m
         clear-footer
         }
      }
   update-runfiles

    set w "$currentport"
    set z [write-file $portfilename $w]
    if {$z!=0} then {
         display-midas-error-report self "" "Error saving port info" $m
         frame-idle
         return
         }

   frame-idle
   }

proc click-config e {
   global basedir soap_port
   global device1selected devlist
   inform soic start $soap_port
   }

proc click-add {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   frame-busy
    if { $currentvolume == ""} return
    if { $currentfile == ""} return
    if { $currentstart == ""} return
    if { $currentend == ""} return
   set l "$currentvolume $currentfile $currentstart $currentend"
   lappend runfilelist [format-runfile $l]
   set runfileselected [expr [llength $runfilelist] - 1]
   paint-runfiles
   frame-idle
   }

proc click-replace {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   frame-busy
   set l "$currentvolume $currentfile $currentstart $currentend"
   set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc init-values {} {
   global debug basedir tracelevel tracelist
   global devlist device1selected device2selected portfilename

   read-handler-directory

    set m ""
   set z [mtsort $basedir "i trace ?" m 1]
   set tracelevel $m
   if {$debug} then {puts "soi init: trace $z $m"}

   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device1selected [lsearch $devlist $m] } \
                        else { set device1selected 0 }
   if {$debug} then {puts "soi init: device1 $z $m"}

   set z [mtsort $basedir "i stream 2" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device2selected [lsearch $devlist $m] } \
                        else { set device2selected 0 }
   if {$debug} then {puts "soi init: device2 $z $m"}

   set portfilename $basedir/.port.cf
   return 0
   }


proc start-handler args {
   global started
   if {![init-values]} then { 
      make-frame
      update idletasks
      read-portinfo
      click-refresh
      set started 1
      }
   }
