#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		soi-TDRConfig.tcl		                #
#									#
#	Author(s):	John Cresswell           			#
#									#
#	Last revision:	24/02/2009					#
#									#
#########################################################################

set frametitle "MTsort TDR Online Sort Input Handler Config"

set currentname     0
set currentlabel    0
set currentflag     0
set currentgroup    0
set currentitem     0

# 0=TDR EB Event  1=Event-by-timestamp jump
set eventmode 0
set clockjump 1000
# 0 = JYU GREAT format   1=Vics GREAT format
set eventformat 0

set runfileheader "Param   Name                                                            Label           Flag            Group        Item"

set z [catch {set env(os)} ostype]
if {$z} {set ostype xxx}
if {$ostype == "SunOS"} {
  set devlist {"Not_Active" "Net" "Other?"}
  } {
  set devlist {"Not_Active" "Net" "Other?"}
  }

set eventmodemenustrings {
    "TDR EB Events"
    "Deduce events by TimeStamp jump"
    "Deduce events by label jump"
    "Deduce events by TimeStamp+label jump"
   }

set eventformatmenustrings {
    "JYU GREAT TDR format Events"
    "Vics TDR format Events"
}

#------------------------------------------------------------------------------

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      ADD                    {click-add}
      INSERT                 {click-insert}
      REPLACE                {click-replace}
      REMOVE                 {click-remove}
      SAVE                   {click-save}

      RUNLIST                {click-runlist $c $v}
      EVENTMODE              {click-eventmode $v}
      EVENTFORMAT            {click-eventformat $v}
      PARAM                  {click-paramname $v}
      LABEL                  {click-label $v}
      FLAG                   {click-flag $v}
      GROUP                  {click-group $v}
      ITEM                   {click-item $v}
      JUMP                   {click-tsjump $v}

      SORT-RUN-FRAME         {click-sort-run-frame}
      MYHELP                   {click-help}
      REFRESH                {click-refresh}

      INPUT-HANDLER          {click-input-handler $v}

      STREAM                 {click-stream $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug env
   global eventmodemenustrings eventformatmenustrings
   global handlerlist handlerselected handlerloaded status
   global devlist debugmenustrings
   global runfilelist runfileheader

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}
   if {$debug} {puts stdout "...devlist=$devlist"}

  set vx1 10c
  set vx2 19c
  set vx3 300
  set twid1 15
  set twid2 6
  set twid3 30

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :button ADD :label "Add"
      :button INSERT :label "Insert"
      :button REPLACE :label "Replace"
      :button REMOVE :label "Remove"
      :button SAVE :label "Save"
      :button SORT-RUN-FRAME :label "Sort Run Frame"
      :button MYHELP :label "Help"
      :button REFRESH :label "Refresh"
   :panel RUNFILES :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10
      :message LIST-HEADER :label "$runfileheader" :bold T
     :next-row T
      :list RUNLIST :rows 20 :list-width 600 :x 0 :exclusive T \
             :strings $runfilelist
     :next-row T
      :message FLAG_INFO :label "Flag values:  Energy/Baseline=0 (Standard Group) ... Trace=1 (Extended group)" :bold T
    [:path FLAG_INFO].msg configure -foreground red
     :next-row T
      :text PARAM :label "Name:" :w $twid3 
      :text LABEL :label "Label:" :w 4
      :text FLAG  :label "Flag:" :w 2
      :text GROUP :label "Group:" :w 4
      :text ITEM  :label "Item:" :w 2
      :next-row T
      :choice-stack EVENTMODE :label "Mode" \
         :valuex 6c :strings $eventmodemenustrings :value 0
      :text JUMP  :label "Jump Clks:" :w $twid2 :x 280
      :next-row T
      :choice-stack EVENTFORMAT :label "Format" \
         :valuex 6c :strings $eventformatmenustrings :value 0

  :panel OPTIONS  :layout L :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 10
      :message INPUT-HANDLER :label "Input handler:    $handlerloaded" :bold T
      :text INPUT-HANDLER-STATUS :label "Status:" :value $status :bold T \
         :w $twid1 :valuex $vx2 :x 300
      [:path INPUT-HANDLER-STATUS].entry configure -bd 0
   :show T
}

proc paint-handler-status {} {
   global status basedir
   set z [mtsort $basedir "i state" status 1]
   :modify INPUT-HANDLER-STATUS :value $status
   }

proc repaint {} {
   global basedir handlerselected handlerloaded
   global devlist device1selected device2selected
   global device2activestates device2active
   global tracelist tracelevel

   check-handler
   :modify INPUT-HANDLER :label "Input handler:    $handlerloaded"
   paint-handler-status

   paint-runfiles
   }

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	set f ""
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	lappend buf $t
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

proc read-runinfo {} {
   global basedir runfilename runfilelist
   global currentmachine currentport eventmode clockjump eventformat
   set runfilelist {}
   set i 0
   set z 0
   set eventmode 0
   while {$z==0} {
      set i [expr $i + 1]
      set z [read-file $runfilename]
      if {$z!=""} then {
	  foreach l $z {
	      set zzz [isComment? $l]
	      if { [isComment? $l] } {
		  if {[string range $l 0 12] == "#by_timestamp"} {
		      set eventmode 1
		      set clockjump [string range $l 14 end]
		  }
		  if {[string range $l 0 4] == "#mode"} {
		      scan $l "%s %d %d" s eventmode clockjump
		  }
		  if {$clockjump == ""} {set clockjump 1000}
		  if {[string range $l 0 6] == "#format"} {
		      scan $l "%s %d" s eventformat
		  }
	      } {
		  set f [format-runfile $l]
		  lappend runfilelist $f
	      }
	  }
      }
   }
   paint-runfiles
}

proc format-runfile {r} {
   scan $r "%s %d %d %d %d" s1 i1 i2 i3 i4
   set r [format "%-32s %8d %6d %10d %6d" $s1 $i1 $i2 $i3 $i4]
   return $r
   }

proc paint-runfiles {} {
   global runfilelist runfileselected eventmode clockjump eventformat
   global currentname currentlabel currentflag currentgroup currentitem
   :modify RUNLIST :empty T :strings $runfilelist :select $runfileselected
   set runfile [lindex $runfilelist $runfileselected]
    if {[string range $runfile 0 0] != "#"} {
	set currentname [first $runfile]
	set currentlabel [second $runfile]
	set currentflag [third $runfile]
	set currentgroup [fourth $runfile]
	set currentitem [fifth $runfile]
	:modify PARAM :value $currentname
	:modify LABEL :value $currentlabel
	:modify FLAG :value $currentflag
	:modify GROUP :value $currentgroup
	:modify ITEM  :value $currentitem
    } {
	:modify PARAM :value ""
	:modify LABEL :value ""
	:modify FLAG  :value ""
	:modify GROUP :value ""
	:modify ITEM  :value ""
    }
    :modify EVENTMODE  :value $eventmode
    :modify JUMP :value $clockjump
    :modify EVENTFORMAT :value $eventformat
}
                 
proc click-stream e {
   global basedir
   global device1selected devlist
   frame-busy
   set device1selected $e
   set d [lindex $devlist $e]
    set m ""
   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device $d" m 1]
   if {$z!=0} then {set-footer $m; set device1selected -1} \
   else {
      clear-footer
      }
   repaint
   frame-idle
   }

proc click-eventmode {s} {
   global eventmode eventmodemenustrings
   set eventmode $s
   }

proc click-eventformat {s} {
   global eventformat eventformatmenustrings
   set eventformat $s
   }

proc click-tsjump {v} {
   global clockjump
   set clockjump $v
   }

proc click-paramname v {
    global currentname currentlabel
    set currentname [string toupper $v]
    if {$currentname == ""} {set currentname $currentlabel}
   }

proc click-label v {
    global currentname currentlabel
    set currentlabel [string toupper $v]
    if {$currentname == ""} {set currentname $v}
   }

proc click-flag v {
    global currentname currentflag
    set currentflag [string toupper $v]
    if {$currentname == ""} {set currentname $v}
   }

proc click-group v {
    global currentgroup currentitem
    set currentgroup [string toupper $v]
    if {$currentitem == ""} {set currentitem 0}
   }

proc click-item v {
   global currentitem
   set currentitem [string toupper $v]
   }

proc click-add {} {
   global runfilelist runfileselected
   global currentname currentlabel currentflag currentgroup currentitem
   frame-busy
    if { $currentname == ""} return
    if { $currentlabel == ""} return
    if { $currentflag == ""} return
    if { $currentgroup == ""} return
    if { $currentitem == ""} return
    set l "$currentname $currentlabel $currentflag $currentgroup $currentitem"
    set l [format-runfile $l]
   lappend runfilelist $l
   set runfileselected [expr [llength $runfilelist] - 1]
   paint-runfiles
   frame-idle
   }

proc click-insert {} {
   global runfilelist runfileselected
   global currentname currentlabel currentgroup currentitem
   frame-busy
    if { $currentname == ""} return
    if { $currentlabel == ""} return
    if { $currentgroup == ""} return
    if { $currentitem == ""} return
   set l "$currentname $currentlabel $currentgroup $currentitem"
    set l [format-runfile $l]
    set runfileselected [incr runfileselected]
   set runfilelist [linsert $runfilelist $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-replace {} {
   global runfilelist runfileselected
   global currentname currentlabel currentflag currentgroup currentitem
   frame-busy
    if { $currentname == ""} return
    if { $currentlabel == ""} return
    if { $currentflag == ""} return
    if { $currentgroup == ""} return
    if { $currentitem == ""} return
   set l "$currentname $currentlabel $currentflag $currentgroup $currentitem"
    set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-remove {} {
   global runfilelist runfileselected
   frame-busy
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected]
   set runfileselected [expr $runfileselected - 1]
   if { $runfileselected < 1 } then {set runfileselected 0}
   paint-runfiles
   frame-idle
   }

proc click-save {} {
   global basedir sortdir runfilelist runfilename eventmode clockjump eventformat
   frame-busy

    set w [list "\#mode $eventmode $clockjump"]
    lappend w "\#format $eventformat"
    
    foreach r $runfilelist {lappend w $r}
    set z [write-file $runfilename $w]
    if {$z!=0} then {
         display-midas-error-report self "" "Error saving runfiles" $m
         frame-idle
         return
         }
   frame-idle
   }

proc click-runlist {c f} {
   global debug
   global runfile runfilelist runfileselected
   global currentname currentlabel currentflag currentgroup currentitem
   if {$debug} then {puts stderr "click-list: $c $f"}
   clear-footer

   case $c in {
      :list-select {
         set runfile $f
	  set currentname [first $runfile]
	  set currentlabel [second $runfile]
	  set currentflag [third $runfile]
	  set currentgroup [fourth $runfile]
	  set currentitem  [fifth $runfile]
         set runfileselected [lsearch $runfilelist $runfile]
        }
     :list-deselect {
        set runfile ""
         set runfileselected 0
        }
     }          
   if {$runfileselected < 1} then { set runfileselected 0 }
   paint-runfiles
   }

proc click-refresh {} {
   frame-busy
   clear-footer
   read-runinfo
   paint-runfiles
   repaint
   frame-idle
   }

proc init-values {} {
   global debug
   global basedir tracelevel tracelist
   global devlist device1selected device2selected
   global runfilename

   read-handler-directory

    set m ""
   set z [mtsort $basedir "i trace ?" m 1]
   set tracelevel $m
   if {$debug} then {puts "soi init: trace $z $m"}

   set z [mtsort $basedir "i stream 1" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device1selected [lsearch $devlist $m] } \
                        else { set device1selected 0 }
   if {$debug} then {puts "soi init: device1 $z $m"}

   set z [mtsort $basedir "i stream 2" m 1]
   set z [mtsort $basedir "i device ?" m 1]
   if {$z==0 && $m!=""} then { set device2selected [lsearch $devlist $m] } \
                        else { set device2selected 0 }
   if {$debug} then {puts "soi init: device2 $z $m"}

   set runfilename $basedir/currentsetup.MTmap
    if {[file exists $runfilename]} {
	file rename -force $runfilename $basedir/.mtsort.cf
    }
   set runfilename $basedir/.mtsort.cf

   return 0
   }


proc start-handler args {
   global started
   if {![init-values]} then { 
      make-frame
      update idletasks
      click-refresh
      set started 1
      }
   }
