#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		soo.tcl (sort output handler setup)		#
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	19/10/04					#
#									#
#########################################################################

set frame SOO
set frametitle "Sort Output Handler Control"
set icontitle "OutputHandler"

set debug 0
set started 0
set sync 0

set handlerdir ""
set basedir ""
set sortdir ""
set runfilename ""

set handlerloaded ""
set handlerprefix "MTsortOutput"
set handlerselected -1
set handlercurrent -1
set handlersource ""
set handlerlist {}
set handlercount 0
set handler ""
set stream 1

#option for output tape filename 0=ignore, 1=change when input changes
# 2=use input filename 4=generate incremental names(swap on file limit)

set output_style   2
set z [catch {get-eg-option mtsort-output-style1} m]
if {!$z} {set output_style $m}

set status "Not Loaded"

set debugmenustrings { Off Minimal More Events MoreEvents All }
set tracelevel 0

set deviceselected 1

set runfile ""
set runfileselected -1
set runfilelist {}
set runfilecount 0

set currentvolume ""
set currentfile   ""
set currentstart  0
set currentend    0

set maxfilesize 2

set filelist {}

#------------------------------------------------------------------------------

proc fopen {name attr} {
    set f 0
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	if {$attr != "r"} {
	display-midas-error-report self "" $m
	}
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { ! [isComment? $t] } { lappend buf $t }
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

#------------------------------------------------------------------------------

proc last p {
   set n [llength $p]
   incr n -1
   return [lindex $p $n]
   }

proc front p {
   set n [llength $p]
   incr n -1
   return [lreplace $p $n $n]
   }

proc paint-handler-status {} {
   global status basedir
   set z [mtsort $basedir "o state" status 1]
   :modify OUTPUT-HANDLER-STATUS :value $status
   }

proc make-handler-list {} {
   global debug handlerprefix
   global handlerlist handlerselected handlercount filelist
   set handlerlist {}; set handlercount 0
   if {$debug>2} then {puts stderr "make-handlerlist: filelist=$filelist"}
   foreach p $filelist {
      set t [first $p]
      if { [string first $handlerprefix $t]==0} then {
	  set t2 [expr [string first ".so" $t] -1]
         lappend handlerlist [string range $t [string length $handlerprefix] $t2]
         incr handlercount
         }
      }
   set handlerlist [lsort $handlerlist]
   lappend handlerlist ""
   if {$debug>2} then {puts stderr "...handlerlist=$handlerlist"}
   }

proc read-handler-directory {} {
   global handlerdir filelist
    set z [catch {eg "(directory :path '$handlerdir' ?files)"} m]
   if {$z!=0} then {
      display-midas-error-report self $m "Attempting to read directory" $handlerdir
      error $m
      }

   set filelist $m
   make-handler-list
   }

proc check-handler {} {
   global basedir stream
   global handlerlist handlercount handlerloaded handlerselected handlerprefix

   read-handler-directory

    set m ""
   set z [mtsort $basedir "o handler $stream ?" m 1]
   if {$z} then {
      set handlerselected -1
      set handlerloaded "No"
      set-footer $m
      } \
   else { 
      set i [string last $handlerprefix $m]
      if {$i != -1} then {
         set handlerloaded [string range $m [expr $i + [string length $handlerprefix]] end]
	  set handlerloaded [string trimright $handlerloaded " "]
         set handlerselected [lsearch $handlerlist $handlerloaded]
         }
      }
   }

proc confirm {m action} {
   global debug
   set i [tk_dialog [self frame-name].quitdialog "[self frame-title] Confirmation" $m  "warning" 1 "Cancel" "Confirm"]
   if {$i} then $action
   if {$debug} then {puts stderr "confirm: $m choice $i"}
   }

proc click-debug e {
   global basedir tracelevel tracelist
   frame-busy
   set tracelevel $e
    set m ""
   set z [mtsort $basedir "o trace $tracelevel" m 1]
   :modify DEBUG :value $tracelevel
   frame-idle
   }

proc click-sort-run-frame {} {
   clear-footer
   inform soc {start "" ""}
   }

proc click-help {} {
    inform help html_show {"" $env(MTSORTBASE)/manual/sortoutput.html}
   }

# Requires htmlviewer.patch.tcl  to work ...
proc click-help {} {
    global env
    inform htmlviewer startfile $env(MTSORTBASE)/manual/sortoutput.html
   }

proc format-runfile {r} {
   scan $r "%s %s %d %d" s1 s2 i1 i2
   set r [format "%6s %-30s %12d %12d" $s1 $s2 $i1 $i2]
   return $r
   }

proc read-runfile {} {
   global debug
   if {$debug} {puts stderr "read-runfile..."}
   send soc call-back soo soo.read-runfile-continue get-sortprogram
   send soc call-back soo soo.read-handlerdir-continue get-handlerdir
   }

proc soo.read-runfile-continue f {
   global debug stream
   global runfilename runfilelist runfileselected
   global sortdir basedir
   if {$debug}  then {puts stderr "[self frame-name]: call-back result $f"}

   set sortdir $f
   set basedir $sortdir
   set runfilename $sortdir/.outruninfo$stream
   set runfileselected 0

   if {![file exists $runfilename]} {
      set-footer "Runfile $runfilename does not exist!"
      return
      }
   }

proc soo.read-handlerdir-continue f {
    global handlerdir
    set handlerdir $f
}

proc load-handler-source {} {
   global handlerloaded handlersource debug

    set type ""
    if {[string last "Null" $handlerloaded] != -1} {set type "Null"}
    if {[string last "Tape" $handlerloaded] != -1} {set type "Tape"}
    if {[string last "Disk" $handlerloaded] != -1} {set type "Disk"}
    if {$type == ""} {set type "Null"}
    set hs soo-$type

   if {$handlersource == $hs} {
       click-refresh
       bring-to-front [self frame-name]
       return
   }
   if {$debug} {puts stderr "Loading source for $hs ..."}
   catch {destroy .soo}
   loadsource $hs
   set started 0
   start-handler
   set started 1
   set handlersource $hs
}

proc click-refresh {} {
   frame-busy
   clear-footer
   read-runinfo
   paint-runfiles
   repaint
   frame-idle
   }

proc click-save {} {
    global basedir stream sortdir runfilelist
    frame-busy
    set i 1
    set z 0
    set m ""
    foreach x $runfilelist {
	set z [mtsort $basedir "o runinfo $stream $i $x" m 1]
	set i [expr $i + 1]
	if {$z!=0} {
	    display-midas-error-report self "" "Error saving runfiles:\n" $m
	    frame-idle
	    return
	}  {
	    clear-footer
	}
    }
    set z [mtsort $basedir "o runinfo $stream $i eof" m 1]
    set-footer $m
    update-runfiles
    frame-idle
}

proc update-runfiles {} {
    global basedir stream runfilelist 

    set r ""
    foreach line $runfilelist {lappend r $line}
    write-file $basedir/.outruninfo$stream $r
}

proc read-runfiles {} {
    global basedir stream runfilelist 

    set runfilelist {}
    set r [read-file $basedir/.outruninfo$stream]
    if {$r!=""} {
	foreach l $r {
	    set f [format-runfile $l]
	    lappend runfilelist $f
	}
    }
}

proc set-runfiles {} {
    global basedir stream runfilelist 

    set i 1
    foreach f $runfilelist {
	set z [mtsort $basedir "o runinfo $stream $i $f" m 1]
	incr i
    }
}

proc click-reget-runfiles {stream} {
    global basedir runfilelist

    read-runfiles
    set-runfiles
    paint-runfiles
#    puts "Output stream $stream: Read runfiles $basedir/.outruninfo$stream"
}

proc click-startblock s {
   global currentstart
   set currentstart  [string toupper $s]
   }

proc click-endblock e {
   global currentend
   set currentend  [string toupper $e]
   }

proc get-filename-option {} {
   global basedir stream output_style

    set z [catch {get-eg-option mtsort-output-style[set stream]} m]
    if {!$z} {
	set output_style $m
	click-filename-option $output_style
    } {
	set m ""
	set z [mtsort $basedir "o style $stream ?" m 1]
	if {$z!=0} then {set-footer $m} \
	    else {clear-footer}
	set output_style $m
	edit-defaults mtsort-output-style[set stream] "set-eg-option mtsort-output-style[set stream] $output_style"
    }
   }

proc paint-filename-option {} {
   global output_style
   :modify FILENAME-OPTION :value $output_style
}

proc click-filename-option v {
   global basedir stream output_style
   set output_style $v
   set m ""
   set z [mtsort $basedir "o style $stream $output_style" m 1]
   if {$z!=0} then {set-footer $m} \
   else {clear-footer}
   paint-filename-option
   edit-defaults mtsort-output-style "set-eg-option mtsort-output-style $output_style"
   }


proc send-output-style {} {
    global basedir stream output_style maxfilesize
   set m ""
   set z [mtsort $basedir "o style $stream $output_style" m 1]
   set m ""
   set z [mtsort $basedir "o maxfile $maxfilesize" m 1]
}

proc finish {} {DeleteTask}

proc start args {
   global started stream handlerselected handlercurrent debug env
    global USE_SOAP soap_port
    
    set stream [first $args]

    set soap_port [second $args]

    if {! $started} {
	if {$soap_port == 0} {
	    set USE_SOAP 0
	    loadsource mtsortcommand
	} {
	    set USE_SOAP 1
	    loadsource mtsortsoap
	    soap-connect
	}
    } {
	if {$soap_port} {soap_connect}
    }


   read-runfile
   check-handler

   if {$handlerselected == -1} {return}
   if {$handlerselected != $handlercurrent} {
      load-handler-source
      set handlercurrent $handlerselected
   } {
      if {$started} then {
	  click-refresh
	  bring-to-front [self frame-name]
      }
   }
}
