#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		sos.tcl (sort setup)				#
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	12/6/97						#
#									#
#########################################################################

set debug 0
set started 0

set frame SOS

set icontitle "Setup"

font create MyFixedFont -family Times -size 13

if { ![info exists env(MTsortVersion)]} {set mtsortversion ""} {set mtsortversion "$env(MTsortVersion)"} 

# Following may be configured explicitly if this code is not appropriate ...

if { ![info exists env(MTSORTBASE)] } {set env(MTSORTBASE) $env(MIDASBASE)/MTsort}

if { ![info exists env(MTSORTDIRPOSTFIX)] } {
    switch $env(os) {
	SunOS    {
	    switch -glob $tcl_platform(machine) {
		i86pc   {set env(MTSORTDIRPOSTFIX) "SunOSx86"}
		default {set env(MTSORTDIRPOSTFIX) "SunOS"}
	    }
	}
	Linux  {
	    switch -glob $tcl_platform(machine) {
		i*86   {set env(MTSORTDIRPOSTFIX) "Linux"}
		*64    {set env(MTSORTDIRPOSTFIX) "Linux64"}
		default {set env(MTSORTDIRPOSTFIX) "Linux"}
	    }
	}
	Linux64  {
	    switch -glob $tcl_platform(machine) {
		*64    {set env(MTSORTDIRPOSTFIX) "Linux64"}
		default {set env(MTSORTDIRPOSTFIX) "Linux"}
	    }
	}
	default {set env(MTSORTDIRPOSTFIX) ""; set-footer "Warning MTSORTPOSTFIX set to Null"}
    }
}
set env(PATH) $env(MTSORTBASE)/bin_[set env(MTSORTDIRPOSTFIX)]$mtsortversion:$env(PATH)
set env(LD_LIBRARY_PATH) $env(MTSORTBASE)/lib_[set env(MTSORTDIRPOSTFIX)]$mtsortversion:$env(LD_LIBRARY_PATH)

# End of configurable section

set sortversion [exec cat $env(MTSORTBASE)/tcl/mtsortversion.tcl]

set frametitle "MTsort Setup Window [set sortversion]"

set currentversion [exec cat $env(MTSORTBASE)/tcl/sos-version.tcl]
set currentversion "[set currentversion] [set env(MTSORTDIRPOSTFIX)]"

set nlistrows 10

set setupstatus UNKNOWN



set preferences(mtsort-sortfilename) ""
set preferences(mtsort-sortfilepath) $env(HOME)/sortfiles
set preferences(mtsort-sortdirpath) $env(HOME)/sortdirs
load_preferences
set sortfile $preferences(mtsort-sortfilename)
set pathnames(0) $preferences(mtsort-sortfilepath)
set pathnames(1) $preferences(mtsort-sortdirpath)

set dirlistname ""
set sortprogram ""

proc modify-setupstatus {} {
    global currentversion pathnames sortprogram
    set filepath $pathnames(1)/$sortprogram
    if {[file exists $filepath/setup_status]} then {
	set setupstatus [exec cat $filepath/setup_status]
	if {[first $setupstatus] == "OK.v4a"} {
	    set setupstatus "[first $currentversion] [second $setupstatus]"
	    exec echo "$setupstatus" >& $filepath/setup_status
	}
    }
}

proc set-userlib {} {
    global pathnames userlib sortfile
    set filepath $pathnames(1)/$sortfile
    if {[file exists $filepath/.userlib]} {
	set userlib [exec cat $filepath/.userlib]
    } {
	set userlib ""
    }
}

set userlib ""
if {$sortfile != ""} set-userlib

set dataformat "eurogam"
set sortfilestrings {}
set nthsortfile 0
set choosestring "T"

set dirlisttype "sort files"
set listtypechoice {"sort files" "sort programs"}
set editstrings ""
foreach editor { gedit textedit emacs xemacs dtpad } {
    set z [ catch {exec which $editor} m ]
    if {!$z && $m!=""} { lappend editstrings $editor }
}

set mapstyle "now"
set configstrings {"Sym4d"}

set z [catch {exec uname} m]
if {$z==0} {set uname $m} {set uname Unknown}

set experimentdir ""
set localexpt 0
set apparatus_name      "sort"
set apparatus_host      "localhost"
set apparatus_port      10235

#------------------------------------------------------------------------------

proc set-sortfilepath {} {
    global pathnames sortfile
    set filepath $pathnames(1)/$sortfile
    catch {exec rm $filepath/.sortfilepath}
    catch {exec echo $pathnames(0) > $filepath/.sortfilepath}
}

proc get-sortfilepath {} {
    global pathnames sortfile sortprogram
    set filepath $pathnames(1)/$sortprogram
    if {[file exists $filepath/.sortfilepath]} {
	set sortfilepath [exec cat $filepath/.sortfilepath]
	set $pathnames(0) sortfilepath
    } {
	set sortfilepath $pathnames(0)
    }
    return $sortfilepath
}

#------------------------------------------------------------------------------

proc click-quit f {
   global env debug started
   if {$debug} then {puts stderr "click-quit with argument \{$f\}" }
   destroy-frame $f
   set started 0
}

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit $f; return}
   switch -glob $w {

      LIST-TYPE      {click-list-type $v}
      FILELIST       {click-filelist $c $v}

      UP             {click-up}
      MYHELP         {click-help}
      DELETE         {click-delete}
      REFRESH        {click-refresh}

      SORTFILEDIR    {click-sortfiledir $v}
      SORTFILE       {click-sortfile $v}

      EDIT-MENU -
      EDIT           {click-edit $v}
      CONFIG         {click-config $v}
      COMPILE        {click-compile}
      RUN            {click-run}
      RUNSHMSAS      {click-run-shmsas}
      POLYEDIT       {click-polyedit}
      GAINMATCH      {click-gainmatch}
      COMPLIST       {click-complist}
      DISMISS        {click-dismiss-complist}
      RECENTCHANGES  {click-recentchanges}
      RCDISMISS      {click-dismiss-recentchanges}
      USERLIB        {click-userlib $v}
      MAP-OPTION     {click-map-option $v}

      default        {puts stdout $arg}
   }
}

proc double-click arg {
   global debug frame

   if {$debug} then {puts stderr "double-click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set w [lindex $arg 2]
   set v [lindex $arg 3]

   switch -glob $w {
        FILELIST     {click-compile}
	default      {puts stdout $arg}
   }
}

proc make-frame {} {
   global debug
   global env frametitle icontitle
   global pathnames sortfile
   global sortfilestrings nthsortfile listtypechoice dirlisttype editstrings
   global userlib configstrings

   set xpos 18c
   set xpos1 24c
   set xpos2 15c
   set xpos3 16c
   set rows 12
   set twidth 36
   set lwidth 200
   if {$debug} then {puts stderr "make-frame"}

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel LOG  :layout B :padx 10 :pady 5
     :layout V; :row-gap 2; :col-gap 20
#      :message SORTLOG-HEADER :label "S O R T    M E S S A G E S" :bold F \
             :valuex 60c
      :galley SORTLOG :rows 12 :cols 85 :x 0 :deaf T
  :panel LISTING  :layout R :padx 10 :pady 5
     :layout H; :row-gap 10; :col-gap 5
      :choice LIST-TYPE   :strings $listtypechoice :value 0 \
             :bold T :x 40
     :next-row T
      :list FILELIST  :rows $rows :list-width $lwidth :x 0 :exclusive T \
	     :strings $sortfilestrings :double-click T
    [:path FILELIST].list configure -font MyFixedFont
     :next-row T
      :button UP      :label "Up"
      :button DELETE  :label "Delete"
      :button REFRESH :label "Refresh"
     :next-row T
      :button RECENTCHANGES :colour aquamarine1 :label "View Recent Changes"
  :panel SETUPFILE  :layout T :padx 10 :pady 10 
     :layout H; :row-gap 10; :col-gap 10
      :text SORTFILEDIR :label "Sortfiles directory:" :value $pathnames(0) \
         :valuex $xpos2 :w $twidth
     :next-row T
      :text SORTFILE :label "Sortfile:" :value $sortfile :valuex $xpos2 :w $twidth
     :next-row T
      :message STATUS :label "Setup status:" :valuex $xpos
      :message STATUSINFO :label UNKNOWN :bold T
  :panel BUTTONS  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10
      :menubutton EDIT :label "Edit sortfile" :strings $editstrings
      :button POLYEDIT :label " Edit polygons "
      :button GAINMATCH :label " Gainmatch window"
     :next-row T
      :button COMPILE :label "Compile sortfile"
      :button RUN :label "Run sort program"
     :next-row T
      :button COMPLIST :label "View compiler messages"
      :checkbox MAP-OPTION :label "Defer map creation to run-time" :value 0 \
         :valuex $xpos1
#      :menubutton CONFIG :label "Command config" :strings $configstrings
   :show T
     :next-row T
      :text USERLIB :label "Optional user library" :value $userlib \
         :valuex $xpos3 :w $twidth
     :next-row T
      :message SHMSTATUS :label "ShmSas status:"
      :message SHMSTATUSINFO :label "Running" :bold T :valuex 10c
      :button RUNSHMSAS :label "Run SHMSAS" :x 220
      :button MYHELP    :label "Help" :x 350

    bind all <Control-Alt-KeyPress-Delete> {click-quit [self frame-name]}

}

proc click-list-type {n} {
   global debug listtypechoice dirlisttype
   if {$debug} then {puts stderr "click-list-type: $n"}
   set dirlisttype [lindex $listtypechoice $n]
   read-directory
   paint-directory
   update-setup-status
}

proc click-filelist {c f} {
   global debug userlib env preferences
   global pathnames sortfile sortprogram dirlisttype
   if {$debug} then {puts stderr "click-filelist: $f"}
   clear-footer
   case $c in {
      :list-select {
	  if {$dirlisttype == "sort files"} {
	      if {$debug} then {puts stderr "...sortfile list"}
	      
	      if {[file isfile $pathnames(0)/$f]} then {
		  set sortfile $f
		  set userlib ""
		  :modify USERLIB :value $userlib
		  :modify SORTFILE :value $sortfile
		  reset-default-sortprogram
		  update-setup-status
		  repaint
	      }  else {
		  if {[file isdir $pathnames(0)/$f]} then {
		      do-set-sortfile-path $pathnames(0)/$f
		  } else {
		      display-midas-error-report self "" "Cannot select sortfile:\n" \
			  "$f is not a file or directory"
		      set sortfile ""
		      update-setup-status
		      repaint
		  }
	      }
	  } else {
	     if {$debug} then {puts stderr "...sortprog list"}
	     
	     set runpath $pathnames(1)/$f
	     if {$debug} then {puts stderr "...runpath: $runpath"}
	     set sortprogram $f
	      set pathnames(0) $env(HOME)/sortfiles
	     if {[file isfile $pathnames(0)/$f]} {set sortfile $f} {
		 set pathnames(0) [get-sortfilepath]
		 if {[file isfile $pathnames(0)/$f]} {set sortfile $f} {
		     set sortfile ""
		 }
	     }
	      :modify SORTFILEDIR :value $pathnames(0)
	     :modify SORTFILE :value $sortfile
	     update-setup-status
	     repaint
         }
      }
     :list-deselect {
         if {$dirlisttype == "sort files"} {
            set sortfile ""
	 } \
         else {
            set sortprogram ""
         }
     }
    }

    set preferences(mtsort-sortfilename) $sortfile
    set preferences(mtsort-sortfilepath) $pathnames(0)
    update_preferences
   }

proc do-set-sortfile-path v {
   global pathnames sortfile dirlisttype sortfilestrings nthsortfile
   set pathnames(0) $v
   :modify SORTFILEDIR :value $pathnames(0)
   if {$dirlisttype=="sort files"} then {
      read-directory
      set sortfile [first $sortfilestrings]
      :modify SORTFILE :value $sortfile
      set nthsortfile 0
      reset-default-sortprogram
      paint-directory
      }
   update-setup-status
   repaint
   }

proc click-up {} {
   global debug
   global frame pathnames env sortfile nthsortfile sortfilestrings
   if {$debug} then {puts stderr "click-up"}
   frame-busy
   clear-footer
   set i [string last / $pathnames(0)]
   if {$i<0} then {return}
   set x ""
   if {$i==0} then {set x "/"}
   if {$i>0} then {set x [string range $pathnames(0) 0 [expr $i-1]]}
# Reset sortfile name from new list read in...
   if {[file exists $pathnames(0)]} then {
      do-set-sortfile-path $x
      }
   frame-idle
   }

proc click-help {} {
    global env
    inform help html_show {"" $env(MTSORTBASE)/manual/sortsetup.html}
   }

# Requires htmlviewer.patch.tcl  to work ...
proc click-help {} {
    global env
    set z [ catch {exec rm -f $env(MIDASTempDir)/CHANGES.html} m ]
    exec cat $env(MTSORTBASE)/manual/htmlheader $env(MTSORTBASE)/manual/ChangeLog $env(MTSORTBASE)/manual/htmlfooter > $env(MIDASTempDir)/CHANGES.html
    inform htmlviewer startfile $env(MTSORTBASE)/manual/sortsetup.html
   }

proc click-delete {} {
   global debug
   global pathnames sortfile sortprogram
   global dirlisttype
   frame-busy
   clear-footer
   if {$dirlisttype == "sort files"} {
      if {![file isfile $pathnames(0)/$sortfile]} then {
         display-midas-error-report self "" \
                 "Attempt to delete sortfile:\n" "$sortfile is not a file!"
         frame-idle
         return
         }
      confirm "  Delete sortfile   $sortfile ?  " complete-delete
      } \
   else {
      set fulldirname $pathnames(1)/$sortprogram
      if {![file isdirectory $fulldirname]} then {
         display-midas-error-report self "" \
                 "Attempt to delete sort program directory:\n" \
                 "$sortprogram is not a directory!"
         frame-idle
         return
         }
      confirm "  Delete sort program  $sortprogram' ?  " complete-delete
      }
   frame-idle
   }

proc click-refresh {} {
   global debug
   if {$debug} then {puts stderr "click-refresh"}
   repaint
   read-directory
   paint-directory
   update-setup-status
   }

proc click-sortfiledir {d} {
   global pathnames dirlisttype
   global sortfile sortfilestrings
   clear-footer
   if {$dirlisttype == "sort files"} then {
      set pathnames(0) $d
      read-directory
      set sortfile [first $sortfilestrings]
      reset-default-sortprogram
      paint-directory
      } \
    else {
	confirm "  Do you really want to change the sort programs directory to $d?  " "global pathnames;set pathnames(1) $d"
      }
#   update-sortfile-setup-status
   }

proc click-sortfile {f} {
   global preferences sortfile dirlisttype
   clear-footer
   set sortfile $f
   if {$dirlisttype == "sort files"} then {
      read-directory
      reset-default-sortprogram
      paint-directory
      }
   update-setup-status

    set preferences(mtsort-sortfilename) $sortfile
    update_preferences
   }

proc click-edit {m} {
   global debug
   if {$debug} then {puts stderr "click-edit: $m"}
    do-edit $m
#   if {$m=="Text Editor"} then {do-edit textedit }
#   if {$m=="Emacs Editor"} then {do-edit emacs }
   }

proc click-config {m} {
   global debug
   if {$debug} then {puts stderr "click-config: $m"}
   inform sosc start $m
   }

proc click-compile {} {
   global debug userlib
   global pathnames sortfile
   if {$debug} then {puts stderr "click-compile"}
   frame-busy
   clear-footer

   set path $pathnames(0)
   set z [catch {
      if {![file isfile $path/$sortfile]} then \
         {error "Sort compilation failed: invalid sortfile name"}
      if {![file exists $path/$sortfile]} then \
         {error "Sort compilation failed: sortfile does not exist"}
      if {![string length $sortfile]} then \
         {error "Sort compilation failed: no sortfile specified"}
      }  result]
   if {$z!=0} then \
      {
      display-midas-error-report self "" \
              "Cannot compile sortfile `$sortfile'\n" $result
      frame-idle
      return
      }

# create sort setup directory if it doesn't exist
   set fulldirname $pathnames(1)/$sortfile
   if {![file exists $fulldirname]} then { \
      if {$debug} then {puts stderr "click-compile: create sort directory"}
      create-dir $pathnames(1) $sortfile
      read-directory
      paint-directory
      if {$userlib != ""} {click-userlib $userlib}
      do-sortsetup
      frame-idle
      return
      }

   if {$debug} then {puts stderr "click-compile: ask confirm overwrite"}
   confirm "  Compiling $sortfile ...\noverwrite sort program  $sortfile ?  " do-sortsetup
   frame-idle
   }

proc click-run {} {
   global debug currentversion
   global pathnames sortprogram setupstatus
   if {$debug} then {puts stderr "click-run"}

# If shmsas not running, try exec and warn user ...

   clear-footer
    if { ! [is-shmsas-running]} {
	click-run-shmsas; after 1000
	if {[is-shmsas-running]} {
	    set-footer "Shmsas started..."
	    set s "Running"
	} {
	    set-footer "Shmsas not running!"
	    set s "NotRunning"
	    set m ""
	    display-midas-error-report self "" \
              "Shmsas not running\nCorrect problem and re-run" $m
	    return
	}
    } {set s  "Running"}
   :modify SHMSTATUSINFO :label $s

   frame-busy
   set-footer "Starting run for sort $sortprogram..."
   update-setup-status
   set z [catch {
      if {$sortprogram==""} then \
         {error "no sort program selected"}
      if {$setupstatus!="OK"} then \
         {error "Setup status should be $currentversion\nbut is $setupstatus; Please re-compile!"}
      if {$debug} then {
         puts stderr "start sort run window"
         puts stderr "path=$pathnames(1) prog=$sortprogram"
         }
       inform soc start $pathnames(1) $sortprogram
      } m]
   if {$z!=0} then {
      display-midas-error-report self "" \
              "Cannot run sort program  $sortprogram:\n" $m
      }
   clear-footer
   frame-idle
   }

proc get-sasport {} {global apparatus_port; return $apparatus_port}

proc click-run-shmsas {} {
    global env mtsortversion apparatus_port
    if {[is-shmsas-running]} {return}
    set f "$env(MTSORTBASE)/bin_[set env(MTSORTDIRPOSTFIX)]$mtsortversion/shmsas -p$apparatus_port"
    set z [catch {exec bash -c $f &} m]

    after 1000
    if {[is-shmsas-running]} {
	set s "Running"
    } {
	set s "NotRunning"
    }
   :modify SHMSTATUSINFO :label $s
}

proc click-userlib {lib} {
   global userlib pathnames sortfile
   if {$lib ==""} {
      set userlib ""
      set path $pathnames(1)
      catch {exec rm $path/$sortfile/.userlib}
      return
   }

    set list [split $lib " "]
    foreach l $list {
	if {![file exists $l  ]} then {
	    midas-warning "User library specified does not exist:\n$l  "
	    return
	}
	if {![file isfile $l  ]} then {
	    midas-warning "User library specified is not a file:\n$l  "
	    return
	}
    }
   set userlib $lib
   set path $pathnames(1)
   catch {exec rm $path/$sortfile/.userlib}
   catch {exec echo $userlib > $path/$sortfile/.userlib}
}

proc click-map-option v {
   global debug
   global mapstyle
   if {$debug} then {puts stderr "click-map-option: $v"}
   if {$v==1} then {set mapstyle "defer"}
   if {$v==0} then {set mapstyle "now"}
   if {$debug} then {puts stderr "...mapstyle=$mapstyle"}
   }

proc click-polyedit {} {
   global debug env
   global pathnames sortfile
   if {$debug} then {puts stderr "click-polyedit"}
   frame-busy
   clear-footer
   if {$sortfile == ""} {
      display-midas-error-report self "" \
        "You haven't selected a sortfile to read the gates from!"
      frame-idle
      return
   }
   inform sortpdb start $pathnames(0)/$sortfile
   frame-idle
   }

proc click-gainmatch {} {
   global debug
   if {$debug} then {puts stderr "click-gainmatch"}
   frame-busy
   clear-footer
   inform gm start
   frame-idle
   }

proc click-complist {} {
   global pathnames sortfile complist_format
   frame-busy
   clear-footer
   set fulldirname $pathnames(1)/$sortfile
   if {![file exists $fulldirname]} then {
      display-midas-error-report self "" \
              "Sort program $sortfile does not have compilation listing"
      frame-idle
      return
      }
   if {![file exists $fulldirname/complist]} then {
      display-midas-error-report self "" \
              "Sort program $sortfile does not have compilation listing"
      frame-idle
      return
      }
   make-complist-frame [exec cat $fulldirname/complist] "Sort compilation listing for $sortfile"
   frame-idle
   }

proc click-dismiss-complist {} {
   destroy [self frame-name].complist
}

proc click-recentchanges {} {
   global env
   frame-busy
   clear-footer
   set fulldirname $env(MTSORTBASE)/manual/ChangeLog
   if {![file exists $fulldirname]} then {
      display-midas-error-report self "" \
              "Recent Changes file not found"
      frame-idle
      return
      }
   make-recentchanges-frame [exec cat $fulldirname] "Recent Changes listing"
   frame-idle
   }

proc click-dismiss-recentchanges {} {
   destroy [self frame-name].recentchanges
}

proc write-file {f v} {
   exec echo "$v" > $f
   }

#------------------------------------------------------------------------------

proc reset-default-sortprogram {} {
   global debug
   global sortfile sortprogram pathnames
   if {$debug} then {puts stderr "reset-default-sortprogram:"}
   set z [catch {eg "(directory :path '$pathnames(1)' ?files)"} m]
   if {$z!=0} then {
      display-midas-error-report self $m \
              "Attempting to read directory" $pathnames(1)
      error $m
      }
   set ll {}
   set ll [lsort [strip-dots [dirnames $m]]]
   set i [lsearch $ll $sortfile]
   if {$i>=0} then {set sortprogram $sortfile} \
              else {set sortprogram ""}
   if {$debug} then {puts stderr "...sortprogram=$sortprogram"}
   }

proc update-setup-status {} {
   global debug setupstatus
   if {$debug} then {puts stderr "update-setup-status"}
   get-setup-status
   :modify STATUSINFO :label $setupstatus   

    if {[is-shmsas-running]} {
	set s "Running"
    } {
	set s "NotRunning"
    }
   :modify SHMSTATUSINFO :label $s
   }

proc get-setup-status {} {
   global debug currentversion uname userlib
   global pathnames sortfile sortprogram setupstatus dirlisttype
   if {$debug} {puts stderr "get-setup-status: pathnames(1)=$pathnames(1)"}
   if {$debug} {puts stderr "             .....sortfile=$sortfile"}
   set setupstatus "UNKNOWN"
   if {$dirlisttype == "sort files"} then {
      set filepath $pathnames(1)/$sortfile
      } \
   else {
      set filepath $pathnames(1)/$sortprogram
      }

    modify-setupstatus
###
###   if {![file isfile $pathnames(0)/$sortfile]} then {return}
###
   if {$debug} {puts stderr "  ...check sort program setup status"}
   if {![file exists $filepath]} then {
      set setupstatus "NOT COMPILED"
      if {$debug} {puts stderr "setupdir does not exist: $filepath"}
      return
      }
#   :modify USERLIB :value $userlib
   if {[file exists $filepath/setup_status]} then {
      if {[file exists $filepath/.userlib]} {
         set userlib [exec cat $filepath/.userlib]
      } {
	  set userlib ""
      }
       :modify USERLIB :value $userlib

      set setupstatus [exec cat $filepath/setup_status]
      if {$debug} {puts stderr "setupdir exists: $filepath, status=$setupstatus"}
       if {$setupstatus==""} then {set setupstatus "UNKNOWN"; return}

      if {$setupstatus=="OK"} then \
         {set setupstatus "Incompatible: old sort version"}

      if {$setupstatus=="FAILED"} then {return}

       if {[second $setupstatus] != [second $currentversion]} {
	   set setupstatus "INVALID-[second $setupstatus]"
	   return
       }

       if {[first $setupstatus] != [first $currentversion]} {
	   set setupstatus "INVALID-[first $setupstatus]"
	   return
       }

      set setupstatus "OK"

      } \
   else {
      if {$debug} {puts stderr "setupdir exists: $filepath, no setupstatus"}
      set setupstatus "UNKNOWN"
      }
   }

proc do-set-sortfile-path v {
   global pathnames sortfile dirlisttype sortfilestrings
   set pathnames(0) $v
   :modify SORTFILEDIR :value $pathnames(0)
   if {$dirlisttype=="sort files"} then {
      read-directory
      set sortfile [first $sortfilestrings]
      :modify SORTFILE :value $sortfile
      reset-default-sortprogram
      paint-directory
      }
   update-setup-status
   repaint
   }

proc repaint {} {
   global sortfile pathnames
#   $frame.t.e1 delete 0 end
#   $frame.t.e1 insert 0 $pathnames(0)
#   $frame.t.e2 delete 0 end
#   $frame.t.e2 insert 0 $sortfile
   }

proc create-dir {path newdir} {
   global errorCode
   set z [catch {exec mkdir $path/$newdir} m]
   if {$z!=0 && $errorCode!="NONE"} then {
      display-midas-error-report self "" \
              "Attempting to create directory $path/$newdir:\n" $m
      error $m
      }
   }

proc read-directory {} {
   global debug
   global pathnames dirlisttype listtypechoice sortfilestrings
# improve error for case where directory does not exist
   if {$debug>1} {puts stderr "read-directory"}
   set listtype [lsearch $listtypechoice $dirlisttype]
   set z [catch {eg "(directory :path '$pathnames($listtype)' ?files)"} m]
   if {$z!=0} then {
      display-midas-error-report self $m \
              "Attempting to read directory" $pathnames($listtype)
      error $m
      }
   set sortfilestrings [lsort [strip-dots [dirnames $m]]]
   }

proc dirnames l {
   set ll {}
   foreach f $l {lappend ll [first $f]}
   return $ll
   }

proc paint-directory {} {
   global debug
   global sortfilestrings choosestring nthsortfile nlistrows
   if {$debug} then \
     {puts stderr "paint-directory: choosestring '$choosestring', nthsortfile '$nthsortfile'"}
   match-selection
   if {$choosestring!="T"} then {
      if {$nthsortfile < $nlistrows} then {set scrollpos 0} \
      else {set scrollpos [expr $nthsortfile - ($nlistrows/2)]}
      if {$debug} {puts stderr "scrollpos=$scrollpos"}
      :modify FILELIST :empty T :strings $sortfilestrings \
         :select $nthsortfile :see $nthsortfile
      } \
   else {
      :modify FILELIST :empty T :strings $sortfilestrings :deselect -1
      }
   }

proc match-selection {} {
   global nthsortfile choosestring 
   global sortfile sortprogram sortfilestrings
   global dirlisttype
   if {$dirlisttype == "sort files"} then {
      set nthsortfile [lsearch $sortfilestrings $sortfile]
      }  \
   else  {
      set nthsortfile [lsearch $sortfilestrings $sortprogram]
      }
   if {$nthsortfile>=0} then {
      update-sort-file-name
      set choosestring F
      } \
   else {set choosestring T}
   }

proc update-sort-file-name {} {
   global debug
   global sortfile
   if {$debug} then \
      {
      puts stderr "update-sort-file-name: sortfile=$sortfile"
      }
   repaint
   }

proc complete-delete {} {
   global debug
   global errorCode
   global pathnames sortfile sortprogram dirlisttype
   global sortfilestrings nthsortfile
   if {$debug} then {
      puts stderr "complete-delete: sortfile[$nthsortfile]=$sortfile"
      }
# set nthsortfile from sortfile name
   match-selection
# delete sortfile/sort prog dir depending on option
   if {$dirlisttype == "sort files"} {
      set target $pathnames(0)/$sortfile
      set z [catch {exec rm $target} m]
      } \
   else {
      set target $pathnames(1)/$sortprogram
      set z [catch {exec rm -r $pathnames(1)/$sortprogram} m]
      }
   if {$z!=0 && $errorCode!="NONE"} then {
      display-midas-error-report self "" "Delete failed for $target:\n" $m
      frame-idle
      } \
   else \
      {
      clear-footer
      read-directory
# reset string to next one in current list
      set sortfile [lindex $sortfilestrings $nthsortfile]
      paint-directory
      update-setup-status
      frame-idle
      set-footer "$target deleted."
      }
   }

proc do-edit {editor args} {
   global debug
   global errorCode
   global pathnames sortfile env
   if {$debug} then {puts stderr "do-edit"}
   frame-busy
   clear-footer
   set path $pathnames(0)
   set z [catch {
      if {![file exists $path/$sortfile]} then \
         {
         if {![file exists $path]} then { \
           if {$debug} then {puts stderr "do-edit: create sortfiles directory"}
           set z [catch {exec mkdir $path} m]
           if {$z!=0 && $errorCode!="NONE"} then {
              display-midas-error-report self "" \
                      "Attempting to create directory $path:\n" $m
              error $m
              }
           }
         set z [catch {exec echo > $path/$sortfile} m]
         if {$z!=0 && $errorCode!="NONE"} then {
            display-midas-error-report self "" \
                    "Attempting to create file $path/$sortfile:\n" $m
            error $m
            }
         click-refresh
         }
###         {error "Sort file edit failed: file does not exist"}
      if {![file isfile $path/$sortfile]} then {
         display-midas-error-report self "" \
                 "Sortfile edit failed:\n" "$path/$sortfile is not a file"
         return
         }
# need to change cwd because of textedit bug under linux (sets dir as cwd)
      if {$args == ""} then \
         {cd $path; exec $editor $sortfile &; cd} \
      else \
         {cd $path; exec $editor $args $sortfile &; cd}
      } result]
   if {$z != 0} then {set-footer $result}
   frame-idle
   }

proc confirm {m action} {
   global debug
   set i [tk_dialog [self frame-name].quitdialog "[self frame-title] Confirmation" $m  "warning" 1 "Cancel" "Confirm"]
   update idletasks
   if {$i} then $action
   if {$debug} then {puts stderr "confirm: $m choice $i"}
   }

proc do-sortsetup {} {

   global debug currentversion uname userlib mapstyle
   global errorCode
   global sortfile pathnames sortfilestrings setupstatus dataformat dirlisttype

   if {$debug} then {puts stderr "do-sortsetup"}
   if {$debug} then {puts stderr "dataformat = $dataformat"}
   set fulldirname $pathnames(1)/$sortfile
# assume setup is going to fail!
   set setupstatus FAILED   

# set map option
   if {$mapstyle == "defer"} {
       exec touch $fulldirname/.mapfiles
   } {
       file delete $fulldirname/.mapfiles
   }

   empty-sortlog
   put-sortlog \
       "SORT SETUP: ------------------  [exec date]  --------------------"
   set-footer "Please wait, Compiling sort program $sortfile ..." 
   set z [catch {
       exec MTsortSetup $pathnames(0)/$sortfile $fulldirname \
	       $dataformat $userlib } result]
#### should also get stderr 
   put-sortlog $result
   set sortdirectory $pathnames(1)/$sortfile
   if {$z!=0 && $errorCode!="NONE"} then \
      {
      set err [lindex $errorCode 2]
      set result "Sort setup for $sortfile failed with return code $err"
      exec echo FAILED >& $sortdirectory/setup_status
      confirm-compiler-error-report "Setup failed" "delete sort program $sortfile?"
      } \
   else \
      {
      set result "Sort setup of $sortfile successful"
      set-sortfilepath
      exec echo "$currentversion" >& $sortdirectory/setup_status
      exec touch $sortdirectory/sortlog
      exec echo $result "(" $currentversion ", " $uname ")" >> $sortdirectory/sortlog
      reset-default-sortprogram
      if {$dirlisttype != "sort files"} then {
         read-directory
         paint-directory
         }
      }
   set-footer $result
# duplicate footer message to sortlog
   put-sortlog $result
### should output parser error messages here
   put-sortlog \
      "---------------------------------------------------------------------------"
   update-setup-status
   repaint
   frame-idle
   }

proc confirm-compiler-error-report {args} {
   global debug
   global env
   frame-busy
   if {$debug} then {puts stderr "confirm-compiler-error-report: $args"}

   set i [tk_dialog [self frame-name].quitdialog "Error report for [self frame-title]" $args "warning" 1 "Cancel" "Confirm"]
   if {$i} then complete-sortprog-delete
   frame-idle
   }

proc empty-sortlog {} {
   :modify SORTLOG :empty T
}

proc put-sortlog text {
   :modify SORTLOG :empty F :append T :strings "$text\n" :see end
   }

proc complete-sortprog-delete {} {
   global debug
   global errorCode
   global pathnames sortfile
   global sortfilestrings nthsortfile
   if {$debug} then {
      puts stderr "complete-sortprog-delete: sortfile($nthsortfile)=$sortfile"
      }
# set nthsortfile from sortfile name
   match-selection
   set target $pathnames(1)/$sortfile
   set z [catch {exec rm -r $target} m]
   if {$z!=0 && $errorCode!="NONE"} then \
      {display-midas-error-report self "" "Delete failed for $target:\n" $m} \
   else \
      {
      clear-footer
      read-directory
# reset string to next one in current list
      set sortfile [lindex $sortfilestrings $nthsortfile]
      paint-directory
      update-setup-status
      }
   }

proc make-complist-frame {list message} {
   global env
   :frame [self frame-name].complist :transient T :label $message
   :font local text n :fmly courier
   :layout B
   :panel BUTTONS  :layout B :padx 120 :pady 0
    :button DISMISS :label Dismiss
   :panel MAIN  :layout B :padx 0 :pady 0
    :galley LISTING  :rows 30 :cols 80 :deaf T
   :modify LISTING :strings $list :see 0
   :show T
}

proc make-recentchanges-frame {list message} {
   global env
   :frame [self frame-name].recentchanges :transient T :label $message
   :font local text n :fmly courier
   :layout B
   :panel BUTTONS  :layout B :padx 120 :pady 0
    :button RCDISMISS :label Dismiss
   :panel MAIN  :layout B :padx 0 :pady 0
    :galley LISTING  :rows 30 :cols 80 :deaf T
   :modify LISTING :strings $list :see 0
   :show T
}

proc call-back {interp proc result} {
   global debug
   if {$debug} then {puts stderr "[self frame-name]: call-back: $interp $proc $result"}
   send $interp $proc [transparent [$result]]
   }

proc get-sortdir {} {
   global pathnames sortfile
   return $pathnames(1)/$sortfile
   }

#-- spectrum resource list ------------

proc is-shmsas-running {} {
    global env
    switch $env(os) {
	SunOS {set flags -ef}
	Linux     -
	default {set flags aux}
    }
    set z [catch {exec /bin/ps $flags | grep shmsas | grep MIDAS | grep $env(USER) | grep -v /MIDAS/sas} m]
   if {[set m] == "child process exited abnormally"} {return 0}
    if {$z} {return 0}
    return 1
}

proc check_spectrum_server {} {

    if { ! [is-shmsas-running]} {
	click-run-shmsas; after 2000
	if {[is-shmsas-running]} {
	    set-footer "Shmsas started..."
	    set s "Running"
	} {
	    set-footer "Shmsas not running!"
	    set s "NotRunning"
	    set m ""
	    display-midas-error-report self "" \
              "Shmsas not running\nCorrect problem and re-run" $m
	    return $s
	}
    } {set s  "Running"}
   :modify SHMSTATUSINFO :label $s

    return $s
}

proc check_experiment_joined {} {
    global env localexpt experimentdir

    set localexpt 0
    
    #Check if experiment already joined

    set z [catch {set exptjoined [inform ex set joined]} m]
    if {[string range $exptjoined 0 2] != "can"} {
	if {$exptjoined} {
	    set z [catch {set expt [inform ex set currentexperiment]} m]
	    set experimentdir $env(EXPERIMENTBASE)
	    if {[string range $experimentdir 0 7] == "/tmp/tcl"} {set localexpt 1}
	    return
	}
    }
    
    #Use local temporary experiment
    
    set localexpt 1
    set experimentdir [file join $env(MIDASTempDir) tcl[pid]  "sort_localhost"]

    puts "check_experiment: $experimentdir"
    if {[file exists $experimentdir]} {
	set env(EXPERIMENTDIR) $experimentdir
	set env(EXPERIMENTBASE) $experimentdir
	return 0
    } {

	set z [ catch { :file mkdir $experimentdir } m ]
	if { $z != 0 } {
	    puts "Setting Experiment directory FAILED"
	    set-footer "Setting Experiment directory FAILED"
	    return
	}
    }

    # Now set the Experiment directory
    set filename "capabilities"
    :experiment new :path $experimentdir :name $filename

    set-footer "Temporary experiment has been created ..."
    
    :experiment connect :path $experimentdir :name $filename
    set env(EXPERIMENTDIR) $experimentdir

    set env(EXPERIMENTBASE) $experimentdir
    after 1000
 
    #--> Now tell the rest of the session what we have done ...
    foreach task [ Get_Task_Names ] {
        if {[inform $task "info procs set-experiment"] == "set-experiment"} {
	    inform $task set-experiment $experimentdir
        }
    }

    set-footer "Temporary experiment has been Joined ... "

    return 1
}

proc write_SOAPServers {} {
    global env SOAPServer
    global apparatus_name apparatus_host apparatus_port

    set directory $env(EXPERIMENTBASE)
    set file SOAPServers

#    attempt to write SOAP server file
    set z [catch {open [file join $directory $file] w} f]
    if {$z != 0} then {return}

    foreach  item  [array names SOAPServer] {
         if {$item == "Dummy"} {continue}
         puts $f "$item $SOAPServer($item)"
    }

    close $f
}

proc process_soapservers {} {
    global SOAPServer apparatus_name apparatus_host apparatus_port
    global env

    # Check shm segments owner ...
    # Get list of all /dev/shm/shmuser files ...
    set userfound 0
    set portfound 0
    set userlist {}
    set portlist {}
    set filelist [read-directory-full /dev/shm]
    foreach f $filelist {
	set ff [ninth $f]
	if {[string range $ff 0 12] == "shmcontrol102"} {
	    append userlist [third $f]
	    append portlist [string range $ff 10 14]
	    if {[third $f] == $env(USER)} {
		set userfound 1
		set portfound [string range $ff 10 14]
	    }
	}
    }

    if {!$userfound} {
	set portfound 10235
	while {[string first $portfound $portlist] >= 0} {incr portfound}
    }

    set apparatus_port $portfound
    
    # Check shmsas running ...

   switch $env(os) {
	SunOS {set flags -ef}
	Linux     -
	default {set flags aux}
    }
    set z [catch {exec /bin/ps $flags | grep shmsas | grep MIDAS} m]

    set sasstring "$apparatus_host:$apparatus_port Spectrum SpectrumService SpectrumService SpectrumClient"

    inform ex read_SOAPServers
    set found 0
    set servers [inform ex array names SOAPServer]
    if {$servers != ""} {
	foreach server $servers {
	    set SOAPServer($server) [inform ex set SOAPServer($server)]
	    set item SOAPServer($server)
	    if {[first $item] != $apparatus_name} {continue}
	    set SOAPServer($server) $sasstring
	    set found 1
	    break
	}
    }
    if {!$found} {set SOAPServer($apparatus_name) $sasstring}
    write_SOAPServers

    inform ex read_SOAPServers
    inform ex auto-join
}    

proc check_spectrum_resource {} {

    check_experiment_joined

    process_soapservers

    return 1
}

#------------------------------------------------------------------------------

proc init-sort-frame {} {
   global debug
   global errorCode
   global started
   global pathnames env
   global sortfile sortfilestrings nthsortfile sosformat dataformat
   global dirlisttype
   if {$debug} then {puts stderr "init-sort-frame"}
   set started 1
   set dataformat "eurogam"

# Initialise errorCode variable in case it is tested and before an error occurs
   set z [catch {error "errorCode initialisation"} m]

   if {$debug} then {puts stderr "sortfiles path=$pathnames(0) sort programs path=$pathnames(1)"}
# create $HOME/sortdirs for sort setup dirs if it doesn't exist
   if {![file exists $pathnames(1)]} then  {create-dir $env(HOME) "sortdirs"}

   if {[file exists $pathnames(0)]} then {
      read-directory

       if {$sortfile == ""} {
	   set sortfile [first $sortfilestrings]
	   set nthsortfile 0
       } {
	   set nthsortfile [lsearch $sortfilestrings $sortfile]
	   if {$nthsortfile == -1} {
	       set sortfile [first $sortfilestrings]
	       set nthsortfile 0
	   }
       }
      reset-default-sortprogram
      paint-directory
      update-setup-status
      }

    # Check if experiment exists ...

    set ret [check_spectrum_resource]
    switch $ret {
	0 {
	    set absent_proc1 "   No experiment/apparatus connected"
	    display-midas-error-report self "" \
		"Sort initialisation error:\n" $absent_proc1
	}
	1 {clear-footer}
	2 {clear-footer}
    }

    # Check if spectrum server is already running ...

    if {[check_spectrum_server] != "Running"} {
	set absent_proc1 "   No spectrum server running"
	display-midas-error-report self "" \
	    "Sort initialisation error:\n" $absent_proc1
    }

   repaint
   }

###  S O R T   S E T U P  functions

proc click-sortlog {} {
   global pathnames sortprogram
   frame-busy
   clear-footer
   set fulldirname $pathnames(1)/$sortprogram
   if {![file exists $fulldirname/sortlog]} then {
      set-footer "Sort program $sortprogram does not have sortlog file"
      frame-idle
      return
      }

   mksoscl [exec cat $fulldirname/sortlog] "Sortlog for $sortprogram"
   }

proc start args {
   global debug
   global started
   if {$debug} {puts stderr "start: [self frame-name] (started=$started)"}
   if {$started} {
       bring-to-front [self frame-name]
   } {
       make-frame
       init-sort-frame
   }
}

tk_focusFollowsMouse
