#########################################################################
#									#
#	MIDAS General sort package					#
#									#
#	File:		sov.tcl (sort variables control)		#
#									#
#	Author(s):	Janet Sampson/John Cresswell			#
#									#
#	Last revision:	05/12/2001					#
#									#
#########################################################################

set frame SOV
set frametitle "Sort Variables Control"
set icontitle "MTsortVars"

set debug 0
set started 0
set sync 0

set basedir ""
set sortdir ""
set varfilename ""
set varfilelist ""

set debug 0
set showmode ALL
#------------------------------------------------------------------------------

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	display-midas-error-report self "" $m
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { ! [isComment? $t] } { lappend buf $t }
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

proc last p {
   set n [llength $p]
   incr n -1
   return [lindex $p $n]
   }

proc front p {
   set n [llength $p]
   incr n -1
   return [lreplace $p $n $n]
   }

proc confirm {m action} {
   global debug
   set i [tk_dialog [self frame-name].quitdialog "[self frame-title] Confirmation" $m  "warning" 1 "Cancel" "Confirm"]
   if {$i} then $action
   if {$debug} then {puts stderr "confirm: $m choice $i"}
   }

#------------------------------------------------------------------------------

proc click-sort-run-frame {} {
   clear-footer
   inform soc {start "" ""}
   }

proc click-help {} {
    inform help html_show {"" $env(MTSORTBASE)/manual/sortvars.html}
   }

# Requires htmlviewer.patch.tcl  to work ...
proc click-help {} {
    global env
    inform htmlviewer startfile $env(MTSORTBASE)/manual/sortvars.html
   }

proc read-varfile {} {
   global debug
   if {$debug} {puts stderr "read-runfile..."}
   send soc call-back sov sov.read-runfile-continue get-sortprogram
   }

proc sov.read-runfile-continue f {
   global debug showmode
   global varfilename varfilelist varlist
   global sortdir basedir
   if {$debug}  then {puts stderr "[self frame-name]: call-back result $f"}

   set sortdir $f
   set basedir $sortdir
   set varfilename $sortdir/initdata.sort

   if {![file exists $varfilename]} then {
      set-footer "Variables file $varfilename does not exist!"
      return
      }

    set newlist ""
    set varlist [read-file $varfilename]
    set varfilelist [filter-list $varlist]
}

proc filter-list {list} {
    global showmode
    set newlist ""
    foreach l $list {
	if {[llength $l] == 4} {set l "$l RW"}
	switch $showmode {
	    RO  {if {[fifth $l] == "RW"} {continue}}
	    RW  {if {[fifth $l] == "RO"} {continue}}
	}
	if {[first $l] == "word"} {lappend newlist $l}
	if {[first $l] == "long"} {lappend newlist $l}
	if {[first $l] == "float"} {lappend newlist $l}
	if {[first $l] == "longlong"} {lappend newlist $l}
    }

    set newlist [lsort -index 1 $newlist]
    return $newlist
}

set word-type 1
set long-type 2
set float-type 3
set longlong-type 4

proc read-varinfo {} {
    global debug varlist varfilelist basedir
    global word-type long-type float-type longlong-type 
    set varfilelist [filter-list $varlist]
    set nrows 0
    set item [lindex $varfilelist $nrows]
    while {$item != ""} {
	global var$nrows
	set t [set [first $item]-type]
	set d [scan [fourth $item] "%x" v]
	set c "e variable 1 [second $item] $t $v"
	set z [mtsort $basedir $c value 1]
	if {$z} {
	    if {$debug} {puts "read-varinfo $nrows: z $z value $value"}
	    set var$nrows 0
	} {
	    if {$value == [second $item]} {
		set var$nrows 0
	    } {
		if {$debug} {puts "read-varinfo $nrows: z $z value $value"}
		set var$nrows $value
	    }
	}
	incr nrows
	set item [lindex $varfilelist $nrows]
    }
}

proc repaint {} {
    global varscanvas varfilelist
    set nrows 0
    set item [lindex $varfilelist $nrows]
    while {$item != ""} {
	keyvalue-update $nrows $item
	incr nrows
	set item [lindex $varfilelist $nrows]
    }
}

proc redraw {} {
    global varfilelist varscanvas
    set nrows 0
    global nn
    set item [lindex $varfilelist $nrows]
    while {$item != ""} {
	keyvalue $nrows $item
	incr nrows
	set item [lindex $varfilelist $nrows]
    }
    update
    if {! $nrows} {set lastbox "0 0 0 0"} {
       incr nrows -1
       set lastbox [$varscanvas bbox var$nrows]
    }
    $varscanvas configure -scrollregion "0 0 [third $lastbox] [fourth $lastbox]"
    set h [fourth $lastbox]
    if {$h >= 400} {
	$varscanvas configure -height 400
	set lastbox [$varscanvas bbox var[incr nrows -1]]
	$varscanvas configure -scrollregion "0 0 [third $lastbox] [fourth $lastbox]"
    } {$varscanvas configure -height $h}
}

proc click-refresh {} {
   frame-busy
   clear-footer
   read-varinfo
   redraw
   repaint
   frame-idle
   }

proc click-global {} {
    global showmode
    set showmode "RW"
    click-refresh
}

proc click-local {} {
    global showmode
    set showmode "RO"
    click-refresh
}

proc click-all {} {
    global showmode
    set showmode "ALL"
    click-refresh
}

proc makeCanvas {frame canvas} {
    set vscroll [scrollbar $frame.vscroll -command "$canvas yview"]

    set canvas [canvas $canvas -height 400 -width 300 \
		    -yscrollcommand "$vscroll set" -yscrollincrement 20] 
    pack $vscroll -side right -fill y
    pack $canvas -fill both -expand 1

    return $canvas
}

proc keyvalue-update {row item} {
    global varscanvas
    set name var$row
    set frame $varscanvas
    global var$row
    if {[fifth $item] == "RO"} {$frame.$name.text configure -state normal}
    $frame.$name.text delete 0 end
    $frame.$name.text insert 0 [set var$row]
    if {[fifth $item] == "RO"} {$frame.$name.text configure -state disabled}
}

proc keyvalue-accept {row} {
    global varscanvas basedir varfilelist
    global word-type long-type float-type longlong-type 

    set item [lindex $varfilelist $row]
    set name var$row
    global var$row
    set new [$varscanvas.$name.text get]
    set var$row $new

    set t [set [first $item]-type]
    set d [scan [fourth $item] "%x" v]
    set c "e variable 2 [second $item] $t $v $new"
    set z [mtsort $basedir $c value 1]
    if {$z} {
	set-footer [first [split $value "\n"]]
    }
}

if {$env(TKVERSION) > 8.4} {
font create MyFixedFont {*}[font configure TkFixedFont] -weight bold
} {
font create MyFixedFont -family "9x15bold" -weight bold
}

proc keyvalue {row item} {
    global varscanvas
    set name var$row
    set frame $varscanvas
    catch {destroy $frame.$name}
    frame $frame.$name -background white
    label $frame.$name.mess1 -width 10 -text [first $item] -anchor w \
	-background white -font MyFixedFont
    pack $frame.$name.mess1 -side left
    label $frame.$name.mess2 -width 22 -text [string tolower [second $item]] -anchor w \
	-background white -font MyFixedFont
    pack $frame.$name.mess2 -side left

    entry $frame.$name.text -width 10 -background MintCream -disabledforeground black
    $frame.$name.text insert 0 [third $item]
    pack $frame.$name.text -side right

    if {[fifth $item] == "RO"} {
	$frame.$name.text configure -state disabled -background "MistyRose1"
    }
    
    bind  $frame.$name.text <Return> "keyvalue-accept $row"
    set h 20
    $varscanvas create window  0 [expr $h * $row] -tags "$name" \
	-window $frame.$name -anchor nw
}

# make the list frame for displaying the files in dir

proc makeVarsFrame {f} {
    global varscanvas varfilelist

    set varscanvas [makeCanvas $f $f.cvars]
    set nrows 0
    global nn
    set item [lindex $varfilelist $nrows]
    while {$item != ""} {
	keyvalue $nrows $item
	incr nrows
	set item [lindex $varfilelist $nrows]
    }
    update
    incr nrows -1
    set lastbox [$varscanvas bbox var$nrows]
    $varscanvas configure -scrollregion "0 0 [third $lastbox] [fourth $lastbox]"
    set h [fourth $lastbox]
    if {$h >= 400} {
	set lastbox [$varscanvas bbox var[incr nrows -1]]
	$varscanvas configure -scrollregion "0 0 [third $lastbox] [fourth $lastbox]"
    } {$varscanvas configure -height $h}
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      SORT-RUN-FRAME         {click-sort-run-frame}
      MYHELP                   {click-help}
      REFRESH                {click-refresh}
      GLOBAL                 {click-global}
      LOCAL                  {click-local}
      ALL                    {click-all}

      RUNLIST                {click-runlist $c $v}

      DEBUG                  {click-debug $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug
   global env
   global status
   global varfilelist

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}

  set vx1 275

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :button GLOBAL :label "Global"
      :button LOCAL :label "Local"
      :button ALL :label "ALL"
      :button REFRESH :label "Refresh"
      :button MYHELP :label "Help" :x $vx1
   :panel VARIABLES :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
      :message LIST-HEADER :label "MTsort Variables" :bold T :x 280
      :list VARLIST :rows 10 :list-width 500 :x 0 :exclusive T \
             :strings $varfilelist :see 0
    makeVarsFrame [:path VARIABLES]
   :show T
}

proc finish {} {DeleteTask}

proc start args {
   global started debug env soap_port

    if {! $started} {
	set soap_port $args
	if {$soap_port == 0} {
	    loadsource mtsortcommand
	} {
	    loadsource mtsortsoap
	    soap-connect
	}
    } {
	if {$soap_port} {soap_connect}
    }


    read-varfile
    if {$started} {
	click-refresh
	bring-to-front [self frame-name]
    } {
	make-frame
	click-refresh
    }
    set started 1
}
